@@ -1,8 +1,9 @@
 """Provide an authentication layer for Open Peer Power."""
+from __future__ import annotations
+
 import asyncio
 from collections import OrderedDict
 from datetime import timedelta
-import logging
 from typing import Any, Dict, List, Optional, Tuple, cast
 
 import jwt
@@ -20,17 +21,24 @@ from .providers import AuthProvider, LoginFlow, auth_provider_from_config
 EVENT_USER_ADDED = "user_added"
 EVENT_USER_REMOVED = "user_removed"
 
-_LOGGER = logging.getLogger(__name__)
 _MfaModuleDict = Dict[str, MultiFactorAuthModule]
 _ProviderKey = Tuple[str, Optional[str]]
 _ProviderDict = Dict[_ProviderKey, AuthProvider]
 
 
+class InvalidAuthError(Exception):
+    """Raised when a authentication error occurs."""
+
+
+class InvalidProvider(Exception):
+    """Authentication provider not found."""
+
+
 async def auth_manager_from_config(
     opp: OpenPeerPower,
     provider_configs: List[Dict[str, Any]],
     module_configs: List[Dict[str, Any]],
-) -> "AuthManager":
+) -> AuthManager:
     """Initialize an auth manager from config.
 
     CORE_CONFIG_SCHEMA will make sure do duplicated auth providers or
@@ -70,7 +78,7 @@ async def auth_manager_from_config(
 class AuthManagerFlowManager(data_entry_flow.FlowManager):
     """Manage authentication flows."""
 
-    def __init__(self, opp: OpenPeerPower, auth_manager: "AuthManager"):
+    def __init__(self, opp: OpenPeerPower, auth_manager: AuthManager):
         """Init auth manager flows."""
         super().__init__(opp)
         self.auth_manager = auth_manager
@@ -98,7 +106,7 @@ class AuthManagerFlowManager(data_entry_flow.FlowManager):
             return result
 
         # we got final result
-        if isinstance(result["data"], models.User):
+        if isinstance(result["data"], models.Credentials):
             result["result"] = result["data"]
             return result
 
@@ -122,11 +130,12 @@ class AuthManagerFlowManager(data_entry_flow.FlowManager):
                 modules = await self.auth_manager.async_get_enabled_mfa(user)
 
                 if modules:
+                    flow.credential = credentials
                     flow.user = user
                     flow.available_mfa_modules = modules
                     return await flow.async_step_select_mfa_module()
 
-        result["result"] = await self.auth_manager.async_get_or_create_user(credentials)
+        result["result"] = credentials
         return result
 
 
@@ -158,7 +167,7 @@ class AuthManager:
         return list(self._mfa_modules.values())
 
     def get_auth_provider(
-        self, provider_type: str, provider_id: str
+        self, provider_type: str, provider_id: Optional[str]
     ) -> Optional[AuthProvider]:
         """Return an auth provider, None if not found."""
         return self._providers.get((provider_type, provider_id))
@@ -215,12 +224,14 @@ class AuthManager:
 
         return user
 
-    async def async_create_user(self, name: str) -> models.User:
+    async def async_create_user(
+        self, name: str, group_ids: Optional[List[str]] = None
+    ) -> models.User:
         """Create a user."""
         kwargs: Dict[str, Any] = {
             "name": name,
             "is_active": True,
-            "group_ids": [GROUP_ID_ADMIN],
+            "group_ids": group_ids or [],
         }
 
         if await self._user_should_be_owner():
@@ -284,6 +295,7 @@ class AuthManager:
         self,
         user: models.User,
         name: Optional[str] = None,
+        is_active: Optional[bool] = None,
         group_ids: Optional[List[str]] = None,
     ) -> None:
         """Update a user."""
@@ -294,6 +306,12 @@ class AuthManager:
             kwargs["group_ids"] = group_ids
         await self._store.async_update_user(user, **kwargs)
 
+        if is_active is not None:
+            if is_active is True:
+                await self.async_activate_user(user)
+            else:
+                await self.async_deactivate_user(user)
+
     async def async_activate_user(self, user: models.User) -> None:
         """Activate a user."""
         await self._store.async_activate_user(user)
@@ -310,9 +328,7 @@ class AuthManager:
 
         if provider is not None and hasattr(provider, "async_will_remove_credentials"):
             # https://github.com/python/mypy/issues/1424
-            await provider.async_will_remove_credentials(  # type: ignore
-                credentials
-            )
+            await provider.async_will_remove_credentials(credentials)  # type: ignore
 
         await self._store.async_remove_credentials(credentials)
 
@@ -362,6 +378,7 @@ class AuthManager:
         client_icon: Optional[str] = None,
         token_type: Optional[str] = None,
         access_token_expiration: timedelta = ACCESS_TOKEN_EXPIRATION,
+        credential: Optional[models.Credentials] = None,
     ) -> models.RefreshToken:
         """Create a new refresh token for a user."""
         if not user.is_active:
@@ -410,6 +427,7 @@ class AuthManager:
             client_icon,
             token_type,
             access_token_expiration,
+            credential,
         )
 
     async def async_get_refresh_token(
@@ -435,6 +453,8 @@ class AuthManager:
         self, refresh_token: models.RefreshToken, remote_ip: Optional[str] = None
     ) -> str:
         """Create a new access token."""
+        self.async_validate_refresh_token(refresh_token, remote_ip)
+
         self._store.async_log_refresh_token_usage(refresh_token, remote_ip)
 
         now = dt_util.utcnow()
@@ -448,6 +468,40 @@ class AuthManager:
             algorithm="HS256",
         ).decode()
 
+    @callback
+    def _async_resolve_provider(
+        self, refresh_token: models.RefreshToken
+    ) -> Optional[AuthProvider]:
+        """Get the auth provider for the given refresh token.
+
+        Raises an exception if the expected provider is no longer available or return
+        None if no provider was expected for this refresh token.
+        """
+        if refresh_token.credential is None:
+            return None
+
+        provider = self.get_auth_provider(
+            refresh_token.credential.auth_provider_type,
+            refresh_token.credential.auth_provider_id,
+        )
+        if provider is None:
+            raise InvalidProvider(
+                f"Auth provider {refresh_token.credential.auth_provider_type}, {refresh_token.credential.auth_provider_id} not available"
+            )
+        return provider
+
+    @callback
+    def async_validate_refresh_token(
+        self, refresh_token: models.RefreshToken, remote_ip: Optional[str] = None
+    ) -> None:
+        """Validate that a refresh token is usable.
+
+        Will raise InvalidAuthError on errors.
+        """
+        provider = self._async_resolve_provider(refresh_token)
+        if provider:
+            provider.async_validate_refresh_token(refresh_token, remote_ip)
+
     async def async_validate_access_token(
         self, token: str
     ) -> Optional[models.RefreshToken]:
