@@ -37,7 +37,7 @@ def is_on(opp, entity_id=None):
             continue
 
         if not hasattr(component, "is_on"):
-            _LOGGER.warning("Integration %s has no is_on method.", domain)
+            _LOGGER.warning("Integration %s has no is_on method", domain)
             continue
 
         if component.is_on(ent_id):
