@@ -1,6 +1,6 @@
 """Alexa capabilities."""
 import logging
-import math
+from typing import List, Optional
 
 from openpeerpower.components import (
     cover,
@@ -8,6 +8,7 @@ from openpeerpower.components import (
     image_processing,
     input_number,
     light,
+    timer,
     vacuum,
 )
 from openpeerpower.components.alarm_control_panel import ATTR_CODE_FORMAT, FORMAT_NUMBER
@@ -26,6 +27,7 @@ from openpeerpower.const import (
     STATE_ALARM_ARMED_CUSTOM_BYPASS,
     STATE_ALARM_ARMED_HOME,
     STATE_ALARM_ARMED_NIGHT,
+    STATE_IDLE,
     STATE_LOCKED,
     STATE_OFF,
     STATE_ON,
@@ -35,6 +37,7 @@ from openpeerpower.const import (
     STATE_UNKNOWN,
     STATE_UNLOCKED,
 )
+from openpeerpower.core import State
 import openpeerpower.util.color as color_util
 import openpeerpower.util.dt as dt_util
 
@@ -43,7 +46,6 @@ from .const import (
     API_THERMOSTAT_MODES,
     API_THERMOSTAT_PRESETS,
     DATE_FORMAT,
-    PERCENTAGE_FAN_MAP,
     Inputs,
 )
 from .errors import UnsupportedProperty
@@ -70,32 +72,32 @@ class AlexaCapability:
 
     supported_locales = {"en-US"}
 
-    def __init__(self, entity, instance=None):
+    def __init__(self, entity: State, instance: Optional[str] = None):
         """Initialize an Alexa capability."""
         self.entity = entity
         self.instance = instance
 
-    def name(self):
+    def name(self) -> str:
         """Return the Alexa API name of this interface."""
         raise NotImplementedError
 
     @staticmethod
-    def properties_supported():
+    def properties_supported() -> List[dict]:
         """Return what properties this entity supports."""
         return []
 
     @staticmethod
-    def properties_proactively_reported():
+    def properties_proactively_reported() -> bool:
         """Return True if properties asynchronously reported."""
         return False
 
     @staticmethod
-    def properties_retrievable():
+    def properties_retrievable() -> bool:
         """Return True if properties can be retrieved."""
         return False
 
     @staticmethod
-    def properties_non_controllable():
+    def properties_non_controllable() -> bool:
         """Return True if non controllable."""
         return None
 
@@ -168,6 +170,11 @@ class AlexaCapability:
         """Return the supportedOperations object."""
         return []
 
+    @staticmethod
+    def camera_stream_configurations():
+        """Applicable only to CameraStreamController."""
+        return None
+
     def serialize_discovery(self):
         """Serialize according to the Discovery API."""
         result = {"type": "AlexaInterface", "interface": self.name(), "version": "3"}
@@ -221,27 +228,44 @@ class AlexaCapability:
         if inputs:
             result["inputs"] = inputs
 
+        camera_stream_configurations = self.camera_stream_configurations()
+        if camera_stream_configurations:
+            result["cameraStreamConfigurations"] = camera_stream_configurations
+
         return result
 
     def serialize_properties(self):
         """Return properties serialized for an API response."""
         for prop in self.properties_supported():
             prop_name = prop["name"]
-            # pylint: disable=assignment-from-no-return
-            prop_value = self.get_property(prop_name)
-            if prop_value is not None:
-                result = {
-                    "name": prop_name,
-                    "namespace": self.name(),
-                    "value": prop_value,
-                    "timeOfSample": dt_util.utcnow().strftime(DATE_FORMAT),
-                    "uncertaintyInMilliseconds": 0,
-                }
-                instance = self.instance
-                if instance is not None:
-                    result["instance"] = instance
+            try:
+                prop_value = self.get_property(prop_name)
+            except UnsupportedProperty:
+                raise
+            except Exception:  # pylint: disable=broad-except
+                _LOGGER.exception(
+                    "Unexpected error getting %s.%s property from %s",
+                    self.name(),
+                    prop_name,
+                    self.entity,
+                )
+                prop_value = None
 
-                yield result
+            if prop_value is None:
+                continue
+
+            result = {
+                "name": prop_name,
+                "namespace": self.name(),
+                "value": prop_value,
+                "timeOfSample": dt_util.utcnow().strftime(DATE_FORMAT),
+                "uncertaintyInMilliseconds": 0,
+            }
+            instance = self.instance
+            if instance is not None:
+                result["instance"] = instance
+
+            yield result
 
 
 class Alexa(AlexaCapability):
@@ -365,6 +389,10 @@ class AlexaPowerController(AlexaCapability):
 
         if self.entity.domain == climate.DOMAIN:
             is_on = self.entity.state != climate.HVAC_MODE_OFF
+        elif self.entity.domain == vacuum.DOMAIN:
+            is_on = self.entity.state == vacuum.STATE_CLEANING
+        elif self.entity.domain == timer.DOMAIN:
+            is_on = self.entity.state != STATE_IDLE
 
         else:
             is_on = self.entity.state != STATE_OFF
@@ -383,10 +411,17 @@ class AlexaLockController(AlexaCapability):
         "en-AU",
         "en-CA",
         "en-GB",
+        "en-IN",
         "en-US",
         "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
+        "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def name(self):
@@ -425,6 +460,7 @@ class AlexaSceneController(AlexaCapability):
 
     supported_locales = {
         "de-DE",
+        "en-AU",
         "en-CA",
         "en-GB",
         "en-IN",
@@ -432,6 +468,7 @@ class AlexaSceneController(AlexaCapability):
         "es-ES",
         "fr-FR",
         "it-IT",
+        "ja-JP",
     }
 
     def __init__(self, entity, supports_deactivation):
@@ -459,8 +496,10 @@ class AlexaBrightnessController(AlexaCapability):
         "en-US",
         "es-ES",
         "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def name(self):
@@ -503,8 +542,10 @@ class AlexaColorController(AlexaCapability):
         "en-US",
         "es-ES",
         "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def name(self):
@@ -552,8 +593,10 @@ class AlexaColorTemperatureController(AlexaCapability):
         "en-US",
         "es-ES",
         "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def name(self):
@@ -624,9 +667,7 @@ class AlexaPercentageController(AlexaCapability):
             raise UnsupportedProperty(name)
 
         if self.entity.domain == fan.DOMAIN:
-            speed = self.entity.attributes.get(fan.ATTR_SPEED)
-
-            return PERCENTAGE_FAN_MAP.get(speed, 0)
+            return self.entity.attributes.get(fan.ATTR_PERCENTAGE) or 0
 
         if self.entity.domain == cover.DOMAIN:
             return self.entity.attributes.get(cover.ATTR_CURRENT_POSITION, 0)
@@ -640,7 +681,18 @@ class AlexaSpeaker(AlexaCapability):
     https://developer.amazon.com/docs/device-apis/alexa-speaker.html
     """
 
-    supported_locales = {"de-DE", "en-AU", "en-CA", "en-GB", "en-IN", "en-US"}
+    supported_locales = {
+        "de-DE",
+        "en-AU",
+        "en-CA",
+        "en-GB",
+        "en-IN",
+        "en-US",
+        "es-ES",
+        "es-MX",
+        "it-IT",
+        "ja-JP",
+    }
 
     def name(self):
         """Return the Alexa API name of this interface."""
@@ -670,11 +722,8 @@ class AlexaSpeaker(AlexaCapability):
             current_level = self.entity.attributes.get(
                 media_player.ATTR_MEDIA_VOLUME_LEVEL
             )
-            try:
-                current = math.floor(int(current_level * 100))
-            except ZeroDivisionError:
-                current = 0
-            return current
+            if current_level is not None:
+                return round(float(current_level) * 100)
 
         if name == "muted":
             return bool(
@@ -690,7 +739,16 @@ class AlexaStepSpeaker(AlexaCapability):
     https://developer.amazon.com/docs/device-apis/alexa-stepspeaker.html
     """
 
-    supported_locales = {"de-DE", "en-AU", "en-CA", "en-GB", "en-IN", "en-US"}
+    supported_locales = {
+        "de-DE",
+        "en-AU",
+        "en-CA",
+        "en-GB",
+        "en-IN",
+        "en-US",
+        "es-ES",
+        "it-IT",
+    }
 
     def name(self):
         """Return the Alexa API name of this interface."""
@@ -724,12 +782,11 @@ class AlexaPlaybackController(AlexaCapability):
             media_player.SUPPORT_STOP: "Stop",
         }
 
-        supported_operations = []
-        for operation in operations:
-            if operation & supported_features:
-                supported_operations.append(operations[operation])
-
-        return supported_operations
+        return [
+            value
+            for operation, value in operations.items()
+            if operation & supported_features
+        ]
 
 
 class AlexaInputController(AlexaCapability):
@@ -749,9 +806,7 @@ class AlexaInputController(AlexaCapability):
         source_list = self.entity.attributes.get(
             media_player.ATTR_INPUT_SOURCE_LIST, []
         )
-        input_list = AlexaInputController.get_valid_inputs(source_list)
-
-        return input_list
+        return AlexaInputController.get_valid_inputs(source_list)
 
     @staticmethod
     def get_valid_inputs(source_list):
@@ -761,7 +816,7 @@ class AlexaInputController(AlexaCapability):
             formatted_source = (
                 source.lower().replace("-", "").replace("_", "").replace(" ", "")
             )
-            if formatted_source in Inputs.VALID_SOURCE_NAME_MAP.keys():
+            if formatted_source in Inputs.VALID_SOURCE_NAME_MAP:
                 input_list.append(
                     {"name": Inputs.VALID_SOURCE_NAME_MAP[formatted_source]}
                 )
@@ -843,7 +898,16 @@ class AlexaContactSensor(AlexaCapability):
     https://developer.amazon.com/docs/device-apis/alexa-contactsensor.html
     """
 
-    supported_locales = {"en-CA", "en-US"}
+    supported_locales = {
+        "de-DE",
+        "en-AU",
+        "en-CA",
+        "en-IN",
+        "en-US",
+        "es-ES",
+        "it-IT",
+        "ja-JP",
+    }
 
     def __init__(self, opp, entity):
         """Initialize the entity."""
@@ -882,7 +946,17 @@ class AlexaMotionSensor(AlexaCapability):
     https://developer.amazon.com/docs/device-apis/alexa-motionsensor.html
     """
 
-    supported_locales = {"en-CA", "en-US"}
+    supported_locales = {
+        "de-DE",
+        "en-AU",
+        "en-CA",
+        "en-IN",
+        "en-US",
+        "es-ES",
+        "it-IT",
+        "ja-JP",
+        "pt-BR",
+    }
 
     def __init__(self, opp, entity):
         """Initialize the entity."""
@@ -932,6 +1006,7 @@ class AlexaThermostatController(AlexaCapability):
         "fr-FR",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def __init__(self, opp, entity):
@@ -1077,9 +1152,7 @@ class AlexaPowerLevelController(AlexaCapability):
             raise UnsupportedProperty(name)
 
         if self.entity.domain == fan.DOMAIN:
-            speed = self.entity.attributes.get(fan.ATTR_SPEED)
-
-            return PERCENTAGE_FAN_MAP.get(speed, None)
+            return self.entity.attributes.get(fan.ATTR_PERCENTAGE) or 0
 
         return None
 
@@ -1090,7 +1163,22 @@ class AlexaSecurityPanelController(AlexaCapability):
     https://developer.amazon.com/docs/device-apis/alexa-securitypanelcontroller.html
     """
 
-    supported_locales = {"en-AU", "en-CA", "en-IN", "en-US"}
+    supported_locales = {
+        "de-DE",
+        "en-AU",
+        "en-CA",
+        "en-GB",
+        "en-IN",
+        "en-US",
+        "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
+        "fr-FR",
+        "it-IT",
+        "ja-JP",
+        "pt-BR",
+    }
 
     def __init__(self, opp, entity):
         """Initialize the entity."""
@@ -1471,7 +1559,7 @@ class AlexaRangeController(AlexaCapability):
             min_value = float(self.entity.attributes[input_number.ATTR_MIN])
             max_value = float(self.entity.attributes[input_number.ATTR_MAX])
             precision = float(self.entity.attributes.get(input_number.ATTR_STEP, 1))
-            unit = self.entity.attributes.get(input_number.ATTR_UNIT_OF_MEASUREMENT)
+            unit = self.entity.attributes.get(ATTR_UNIT_OF_MEASUREMENT)
 
             self._resource = AlexaPresetResource(
                 ["Value", AlexaGlobalCatalog.SETTING_PRESET],
@@ -1585,6 +1673,7 @@ class AlexaToggleController(AlexaCapability):
         "fr-FR",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def __init__(self, entity, instance, non_controllable=False):
@@ -1641,7 +1730,21 @@ class AlexaChannelController(AlexaCapability):
     https://developer.amazon.com/docs/device-apis/alexa-channelcontroller.html
     """
 
-    supported_locales = {"de-DE", "en-AU", "en-CA", "en-GB", "en-IN", "en-US"}
+    supported_locales = {
+        "de-DE",
+        "en-AU",
+        "en-CA",
+        "en-GB",
+        "en-IN",
+        "en-US",
+        "es-ES",
+        "es-MX",
+        "fr-FR",
+        "hi-IN",
+        "it-IT",
+        "ja-JP",
+        "pt-BR",
+    }
 
     def name(self):
         """Return the Alexa API name of this interface."""
@@ -1654,7 +1757,22 @@ class AlexaDoorbellEventSource(AlexaCapability):
     https://developer.amazon.com/docs/device-apis/alexa-doorbelleventsource.html
     """
 
-    supported_locales = {"en-US"}
+    supported_locales = {
+        "de-DE",
+        "en-AU",
+        "en-CA",
+        "en-GB",
+        "en-IN",
+        "en-US",
+        "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
+        "fr-FR",
+        "hi-IN",
+        "it-IT",
+        "ja-JP",
+    }
 
     def name(self):
         """Return the Alexa API name of this interface."""
@@ -1671,7 +1789,7 @@ class AlexaPlaybackStateReporter(AlexaCapability):
     https://developer.amazon.com/docs/device-apis/alexa-playbackstatereporter.html
     """
 
-    supported_locales = {"de-DE", "en-GB", "en-US", "fr-FR"}
+    supported_locales = {"de-DE", "en-GB", "en-US", "es-MX", "fr-FR"}
 
     def name(self):
         """Return the Alexa API name of this interface."""
@@ -1709,7 +1827,7 @@ class AlexaSeekController(AlexaCapability):
     https://developer.amazon.com/docs/device-apis/alexa-seekcontroller.html
     """
 
-    supported_locales = {"de-DE", "en-GB", "en-US"}
+    supported_locales = {"de-DE", "en-GB", "en-US", "es-MX"}
 
     def name(self):
         """Return the Alexa API name of this interface."""
@@ -1781,7 +1899,14 @@ class AlexaEqualizerController(AlexaCapability):
     https://developer.amazon.com/en-US/docs/alexa/device-apis/alexa-equalizercontroller.html
     """
 
-    supported_locales = {"en-US"}
+    supported_locales = {"de-DE", "en-IN", "en-US", "es-ES", "it-IT", "ja-JP", "pt-BR"}
+    VALID_SOUND_MODES = {
+        "MOVIE",
+        "MUSIC",
+        "NIGHT",
+        "SPORT",
+        "TV",
+    }
 
     def name(self):
         """Return the Alexa API name of this interface."""
@@ -1800,34 +1925,34 @@ class AlexaEqualizerController(AlexaCapability):
             raise UnsupportedProperty(name)
 
         sound_mode = self.entity.attributes.get(media_player.ATTR_SOUND_MODE)
-        if sound_mode and sound_mode.upper() in (
-            "MOVIE",
-            "MUSIC",
-            "NIGHT",
-            "SPORT",
-            "TV",
-        ):
+        if sound_mode and sound_mode.upper() in self.VALID_SOUND_MODES:
             return sound_mode.upper()
 
         return None
 
     def configurations(self):
-        """Return the sound modes supported in the configurations object.
-
-        Valid Values for modes are: MOVIE, MUSIC, NIGHT, SPORT, TV.
-        """
+        """Return the sound modes supported in the configurations object."""
         configurations = None
-        sound_mode_list = self.entity.attributes.get(media_player.ATTR_SOUND_MODE_LIST)
-        if sound_mode_list:
-            supported_sound_modes = []
-            for sound_mode in sound_mode_list:
-                if sound_mode.upper() in ("MOVIE", "MUSIC", "NIGHT", "SPORT", "TV"):
-                    supported_sound_modes.append({"name": sound_mode.upper()})
-
+        supported_sound_modes = self.get_valid_inputs(
+            self.entity.attributes.get(media_player.ATTR_SOUND_MODE_LIST, [])
+        )
+        if supported_sound_modes:
             configurations = {"modes": {"supported": supported_sound_modes}}
 
         return configurations
 
+    @classmethod
+    def get_valid_inputs(cls, sound_mode_list):
+        """Return list of supported inputs."""
+        input_list = []
+        for sound_mode in sound_mode_list:
+            sound_mode = sound_mode.upper()
+
+            if sound_mode in cls.VALID_SOUND_MODES:
+                input_list.append({"name": sound_mode})
+
+        return input_list
+
 
 class AlexaTimeHoldController(AlexaCapability):
     """Implements Alexa.TimeHoldController.
@@ -1853,3 +1978,41 @@ class AlexaTimeHoldController(AlexaCapability):
         When false, Alexa does not send the Resume directive.
         """
         return {"allowRemoteResume": self._allow_remote_resume}
+
+
+class AlexaCameraStreamController(AlexaCapability):
+    """Implements Alexa.CameraStreamController.
+
+    https://developer.amazon.com/docs/device-apis/alexa-camerastreamcontroller.html
+    """
+
+    supported_locales = {
+        "de-DE",
+        "en-AU",
+        "en-CA",
+        "en-GB",
+        "en-IN",
+        "en-US",
+        "es-ES",
+        "fr-FR",
+        "hi-IN",
+        "it-IT",
+        "ja-JP",
+        "pt-BR",
+    }
+
+    def name(self):
+        """Return the Alexa API name of this interface."""
+        return "Alexa.CameraStreamController"
+
+    def camera_stream_configurations(self):
+        """Return cameraStreamConfigurations object."""
+        return [
+            {
+                "protocols": ["HLS"],
+                "resolutions": [{"width": 1280, "height": 720}],
+                "authorizationTypes": ["NONE"],
+                "videoCodecs": ["H264"],
+                "audioCodecs": ["AAC"],
+            }
+        ]
