@@ -115,23 +115,24 @@ Result will be a long-lived access token:
 
 """
 from datetime import timedelta
-import logging
+from typing import Union
 import uuid
 
 from aiohttp import web
 import voluptuous as vol
 
+from openpeerpower.auth import InvalidAuthError
 from openpeerpower.auth.models import (
     TOKEN_TYPE_LONG_LIVED_ACCESS_TOKEN,
     Credentials,
     User,
 )
 from openpeerpower.components import websocket_api
-from openpeerpower.components.http import KEY_REAL_IP
 from openpeerpower.components.http.auth import async_sign_path
 from openpeerpower.components.http.ban import log_invalid_auth
 from openpeerpower.components.http.data_validator import RequestDataValidator
 from openpeerpower.components.http.view import OpenPeerPowerView
+from openpeerpower.const import HTTP_BAD_REQUEST, HTTP_FORBIDDEN, HTTP_OK
 from openpeerpower.core import OpenPeerPower, callback
 from openpeerpower.loader import bind_opp
 from openpeerpower.util import dt as dt_util
@@ -179,13 +180,13 @@ SCHEMA_WS_SIGN_PATH = websocket_api.BASE_COMMAND_MESSAGE_SCHEMA.extend(
 RESULT_TYPE_CREDENTIALS = "credentials"
 RESULT_TYPE_USER = "user"
 
-_LOGGER = logging.getLogger(__name__)
-
 
 @bind_opp
-def create_auth_code(opp, client_id: str, user: User) -> str:
+def create_auth_code(
+    opp, client_id: str, credential_or_user: Union[Credentials, User]
+) -> str:
     """Create an authorization code to fetch tokens."""
-    return opp.data[DOMAIN](client_id, user)
+    return opp.data[DOMAIN](client_id, credential_or_user)
 
 
 async def async_setup(opp, config):
@@ -231,9 +232,9 @@ class TokenView(OpenPeerPowerView):
     requires_auth = False
     cors_allowed = True
 
-    def __init__(self, retrieve_user):
+    def __init__(self, retrieve_auth):
         """Initialize the token view."""
-        self._retrieve_user = retrieve_user
+        self._retrieve_auth = retrieve_auth
 
     @log_invalid_auth
     async def post(self, request):
@@ -251,16 +252,14 @@ class TokenView(OpenPeerPowerView):
             return await self._async_handle_revoke_token(opp, data)
 
         if grant_type == "authorization_code":
-            return await self._async_handle_auth_code(
-                opp, data, str(request[KEY_REAL_IP])
-            )
+            return await self._async_handle_auth_code(opp, data, request.remote)
 
         if grant_type == "refresh_token":
-            return await self._async_handle_refresh_token(
-                opp, data, str(request[KEY_REAL_IP])
-            )
+            return await self._async_handle_refresh_token(opp, data, request.remote)
 
-        return self.json({"error": "unsupported_grant_type"}, status_code=400)
+        return self.json(
+            {"error": "unsupported_grant_type"}, status_code=HTTP_BAD_REQUEST
+        )
 
     async def _async_handle_revoke_token(self, opp, data):
         """Handle revoke token request."""
@@ -271,15 +270,15 @@ class TokenView(OpenPeerPowerView):
         token = data.get("token")
 
         if token is None:
-            return web.Response(status=200)
+            return web.Response(status=HTTP_OK)
 
         refresh_token = await opp.auth.async_get_refresh_token_by_token(token)
 
         if refresh_token is None:
-            return web.Response(status=200)
+            return web.Response(status=HTTP_OK)
 
         await opp.auth.async_remove_refresh_token(refresh_token)
-        return web.Response(status=200)
+        return web.Response(status=HTTP_OK)
 
     async def _async_handle_auth_code(self, opp, data, remote_addr):
         """Handle authorization code request."""
@@ -287,7 +286,7 @@ class TokenView(OpenPeerPowerView):
         if client_id is None or not indieauth.verify_client_id(client_id):
             return self.json(
                 {"error": "invalid_request", "error_description": "Invalid client id"},
-                status_code=400,
+                status_code=HTTP_BAD_REQUEST,
             )
 
         code = data.get("code")
@@ -295,28 +294,37 @@ class TokenView(OpenPeerPowerView):
         if code is None:
             return self.json(
                 {"error": "invalid_request", "error_description": "Invalid code"},
-                status_code=400,
+                status_code=HTTP_BAD_REQUEST,
             )
 
-        user = self._retrieve_user(client_id, RESULT_TYPE_USER, code)
+        credential = self._retrieve_auth(client_id, RESULT_TYPE_CREDENTIALS, code)
 
-        if user is None or not isinstance(user, User):
+        if credential is None or not isinstance(credential, Credentials):
             return self.json(
                 {"error": "invalid_request", "error_description": "Invalid code"},
-                status_code=400,
+                status_code=HTTP_BAD_REQUEST,
             )
 
-        # refresh user
-        user = await opp.auth.async_get_user(user.id)
+        user = await opp.auth.async_get_or_create_user(credential)
 
         if not user.is_active:
             return self.json(
                 {"error": "access_denied", "error_description": "User is not active"},
-                status_code=403,
+                status_code=HTTP_FORBIDDEN,
             )
 
-        refresh_token = await opp.auth.async_create_refresh_token(user, client_id)
-        access_token = opp.auth.async_create_access_token(refresh_token, remote_addr)
+        refresh_token = await opp.auth.async_create_refresh_token(
+            user, client_id, credential=credential
+        )
+        try:
+            access_token = opp.auth.async_create_access_token(
+                refresh_token, remote_addr
+            )
+        except InvalidAuthError as exc:
+            return self.json(
+                {"error": "access_denied", "error_description": str(exc)},
+                status_code=HTTP_FORBIDDEN,
+            )
 
         return self.json(
             {
@@ -335,23 +343,31 @@ class TokenView(OpenPeerPowerView):
         if client_id is not None and not indieauth.verify_client_id(client_id):
             return self.json(
                 {"error": "invalid_request", "error_description": "Invalid client id"},
-                status_code=400,
+                status_code=HTTP_BAD_REQUEST,
             )
 
         token = data.get("refresh_token")
 
         if token is None:
-            return self.json({"error": "invalid_request"}, status_code=400)
+            return self.json({"error": "invalid_request"}, status_code=HTTP_BAD_REQUEST)
 
         refresh_token = await opp.auth.async_get_refresh_token_by_token(token)
 
         if refresh_token is None:
-            return self.json({"error": "invalid_grant"}, status_code=400)
+            return self.json({"error": "invalid_grant"}, status_code=HTTP_BAD_REQUEST)
 
         if refresh_token.client_id != client_id:
-            return self.json({"error": "invalid_request"}, status_code=400)
+            return self.json({"error": "invalid_request"}, status_code=HTTP_BAD_REQUEST)
 
-        access_token = opp.auth.async_create_access_token(refresh_token, remote_addr)
+        try:
+            access_token = opp.auth.async_create_access_token(
+                refresh_token, remote_addr
+            )
+        except InvalidAuthError as exc:
+            return self.json(
+                {"error": "access_denied", "error_description": str(exc)},
+                status_code=HTTP_FORBIDDEN,
+            )
 
         return self.json(
             {
@@ -385,7 +401,7 @@ class LinkUserView(OpenPeerPowerView):
         )
 
         if credentials is None:
-            return self.json_message("Invalid code", status_code=400)
+            return self.json_message("Invalid code", status_code=HTTP_BAD_REQUEST)
 
         await opp.auth.async_link_user(user, credentials)
         return self.json_message("User linked")
@@ -487,7 +503,12 @@ async def websocket_create_long_lived_access_token(
         access_token_expiration=timedelta(days=msg["lifespan"]),
     )
 
-    access_token = opp.auth.async_create_access_token(refresh_token)
+    try:
+        access_token = opp.auth.async_create_access_token(refresh_token)
+    except InvalidAuthError as exc:
+        return websocket_api.error_message(
+            msg["id"], websocket_api.const.ERR_UNAUTHORIZED, str(exc)
+        )
 
     connection.send_message(websocket_api.result_message(msg["id"], access_token))
 
