@@ -2,8 +2,10 @@
 import asyncio
 from datetime import timedelta
 import hashlib
-from typing import Any, List, Sequence
+from types import ModuleType
+from typing import Any, Callable, Dict, List, Optional, Sequence
 
+import attr
 import voluptuous as vol
 
 from openpeerpower import util
@@ -25,35 +27,341 @@ from openpeerpower.const import (
 )
 from openpeerpower.core import callback
 from openpeerpower.exceptions import OpenPeerPowerError
+from openpeerpower.helpers import config_per_platform, discovery
 import openpeerpower.helpers.config_validation as cv
 from openpeerpower.helpers.entity_registry import async_get_registry
+from openpeerpower.helpers.event import (
+    async_track_time_interval,
+    async_track_utc_time_change,
+)
 from openpeerpower.helpers.restore_state import RestoreEntity
-from openpeerpower.helpers.typing import GPSType, OpenPeerPowerType
-import openpeerpower.util.dt as dt_util
+from openpeerpower.helpers.typing import ConfigType, GPSType, OpenPeerPowerType
+from openpeerpower.setup import async_prepare_setup_platform
+from openpeerpower.util import dt as dt_util
 from openpeerpower.util.yaml import dump
 
 from .const import (
+    ATTR_ATTRIBUTES,
     ATTR_BATTERY,
+    ATTR_CONSIDER_HOME,
+    ATTR_DEV_ID,
+    ATTR_GPS,
     ATTR_HOST_NAME,
+    ATTR_LOCATION_NAME,
     ATTR_MAC,
     ATTR_SOURCE_TYPE,
-    CONF_AWAY_HIDE,
     CONF_CONSIDER_HOME,
     CONF_NEW_DEVICE_DEFAULTS,
+    CONF_SCAN_INTERVAL,
     CONF_TRACK_NEW,
-    DEFAULT_AWAY_HIDE,
     DEFAULT_CONSIDER_HOME,
     DEFAULT_TRACK_NEW,
     DOMAIN,
-    ENTITY_ID_FORMAT,
     LOGGER,
+    PLATFORM_TYPE_LEGACY,
+    SCAN_INTERVAL,
+    SOURCE_TYPE_BLUETOOTH,
+    SOURCE_TYPE_BLUETOOTH_LE,
     SOURCE_TYPE_GPS,
+    SOURCE_TYPE_ROUTER,
+)
+
+SERVICE_SEE = "see"
+
+SOURCE_TYPES = (
+    SOURCE_TYPE_GPS,
+    SOURCE_TYPE_ROUTER,
+    SOURCE_TYPE_BLUETOOTH,
+    SOURCE_TYPE_BLUETOOTH_LE,
+)
+
+NEW_DEVICE_DEFAULTS_SCHEMA = vol.Any(
+    None,
+    vol.Schema({vol.Optional(CONF_TRACK_NEW, default=DEFAULT_TRACK_NEW): cv.boolean}),
+)
+PLATFORM_SCHEMA = cv.PLATFORM_SCHEMA.extend(
+    {
+        vol.Optional(CONF_SCAN_INTERVAL): cv.time_period,
+        vol.Optional(CONF_TRACK_NEW): cv.boolean,
+        vol.Optional(CONF_CONSIDER_HOME, default=DEFAULT_CONSIDER_HOME): vol.All(
+            cv.time_period, cv.positive_timedelta
+        ),
+        vol.Optional(CONF_NEW_DEVICE_DEFAULTS, default={}): NEW_DEVICE_DEFAULTS_SCHEMA,
+    }
+)
+PLATFORM_SCHEMA_BASE = cv.PLATFORM_SCHEMA_BASE.extend(PLATFORM_SCHEMA.schema)
+
+SERVICE_SEE_PAYLOAD_SCHEMA = vol.Schema(
+    vol.All(
+        cv.has_at_least_one_key(ATTR_MAC, ATTR_DEV_ID),
+        {
+            ATTR_MAC: cv.string,
+            ATTR_DEV_ID: cv.string,
+            ATTR_HOST_NAME: cv.string,
+            ATTR_LOCATION_NAME: cv.string,
+            ATTR_GPS: cv.gps,
+            ATTR_GPS_ACCURACY: cv.positive_int,
+            ATTR_BATTERY: cv.positive_int,
+            ATTR_ATTRIBUTES: dict,
+            ATTR_SOURCE_TYPE: vol.In(SOURCE_TYPES),
+            ATTR_CONSIDER_HOME: cv.time_period,
+            # Temp workaround for iOS app introduced in 0.65
+            vol.Optional("battery_status"): str,
+            vol.Optional("hostname"): str,
+        },
+    )
 )
 
 YAML_DEVICES = "known_devices.yaml"
 EVENT_NEW_DEVICE = "device_tracker_new_device"
 
 
+def see(
+    opp: OpenPeerPowerType,
+    mac: str = None,
+    dev_id: str = None,
+    host_name: str = None,
+    location_name: str = None,
+    gps: GPSType = None,
+    gps_accuracy=None,
+    battery: int = None,
+    attributes: dict = None,
+):
+    """Call service to notify you see device."""
+    data = {
+        key: value
+        for key, value in (
+            (ATTR_MAC, mac),
+            (ATTR_DEV_ID, dev_id),
+            (ATTR_HOST_NAME, host_name),
+            (ATTR_LOCATION_NAME, location_name),
+            (ATTR_GPS, gps),
+            (ATTR_GPS_ACCURACY, gps_accuracy),
+            (ATTR_BATTERY, battery),
+        )
+        if value is not None
+    }
+    if attributes:
+        data[ATTR_ATTRIBUTES] = attributes
+    opp.services.call(DOMAIN, SERVICE_SEE, data)
+
+
+async def async_setup_integration(opp: OpenPeerPowerType, config: ConfigType) -> None:
+    """Set up the legacy integration."""
+    tracker = await get_tracker(opp, config)
+
+    legacy_platforms = await async_extract_config(opp, config)
+
+    setup_tasks = [
+        asyncio.create_task(legacy_platform.async_setup_legacy(opp, tracker))
+        for legacy_platform in legacy_platforms
+    ]
+
+    if setup_tasks:
+        await asyncio.wait(setup_tasks)
+
+    async def async_platform_discovered(p_type, info):
+        """Load a platform."""
+        platform = await async_create_platform_type(opp, config, p_type, {})
+
+        if platform is None or platform.type != PLATFORM_TYPE_LEGACY:
+            return
+
+        await platform.async_setup_legacy(opp, tracker, info)
+
+    discovery.async_listen_platform(opp, DOMAIN, async_platform_discovered)
+
+    # Clean up stale devices
+    async_track_utc_time_change(opp, tracker.async_update_stale, second=range(0, 60, 5))
+
+    async def async_see_service(call):
+        """Service to see a device."""
+        # Temp workaround for iOS, introduced in 0.65
+        data = dict(call.data)
+        data.pop("hostname", None)
+        data.pop("battery_status", None)
+        await tracker.async_see(**data)
+
+    opp.services.async_register(
+        DOMAIN, SERVICE_SEE, async_see_service, SERVICE_SEE_PAYLOAD_SCHEMA
+    )
+
+    # restore
+    await tracker.async_setup_tracked_device()
+
+
+@attr.s
+class DeviceTrackerPlatform:
+    """Class to hold platform information."""
+
+    LEGACY_SETUP = (
+        "async_get_scanner",
+        "get_scanner",
+        "async_setup_scanner",
+        "setup_scanner",
+    )
+
+    name: str = attr.ib()
+    platform: ModuleType = attr.ib()
+    config: Dict = attr.ib()
+
+    @property
+    def type(self):
+        """Return platform type."""
+        for methods, platform_type in ((self.LEGACY_SETUP, PLATFORM_TYPE_LEGACY),):
+            for meth in methods:
+                if hasattr(self.platform, meth):
+                    return platform_type
+
+        return None
+
+    async def async_setup_legacy(self, opp, tracker, discovery_info=None):
+        """Set up a legacy platform."""
+        LOGGER.info("Setting up %s.%s", DOMAIN, self.type)
+        try:
+            scanner = None
+            setup = None
+            if hasattr(self.platform, "async_get_scanner"):
+                scanner = await self.platform.async_get_scanner(
+                    opp, {DOMAIN: self.config}
+                )
+            elif hasattr(self.platform, "get_scanner"):
+                scanner = await opp.async_add_executor_job(
+                    self.platform.get_scanner, opp, {DOMAIN: self.config}
+                )
+            elif hasattr(self.platform, "async_setup_scanner"):
+                setup = await self.platform.async_setup_scanner(
+                    opp, self.config, tracker.async_see, discovery_info
+                )
+            elif hasattr(self.platform, "setup_scanner"):
+                setup = await opp.async_add_executor_job(
+                    self.platform.setup_scanner,
+                    opp,
+                    self.config,
+                    tracker.see,
+                    discovery_info,
+                )
+            else:
+                raise OpenPeerPowerError("Invalid legacy device_tracker platform.")
+
+            if scanner:
+                async_setup_scanner_platform(
+                    opp, self.config, scanner, tracker.async_see, self.type
+                )
+                return
+
+            if not setup:
+                LOGGER.error("Error setting up platform %s", self.type)
+                return
+
+        except Exception:  # pylint: disable=broad-except
+            LOGGER.exception("Error setting up platform %s", self.type)
+
+
+async def async_extract_config(opp, config):
+    """Extract device tracker config and split between legacy and modern."""
+    legacy = []
+
+    for platform in await asyncio.gather(
+        *(
+            async_create_platform_type(opp, config, p_type, p_config)
+            for p_type, p_config in config_per_platform(config, DOMAIN)
+        )
+    ):
+        if platform is None:
+            continue
+
+        if platform.type == PLATFORM_TYPE_LEGACY:
+            legacy.append(platform)
+        else:
+            raise ValueError(
+                f"Unable to determine type for {platform.name}: {platform.type}"
+            )
+
+    return legacy
+
+
+async def async_create_platform_type(
+    opp, config, p_type, p_config
+) -> Optional[DeviceTrackerPlatform]:
+    """Determine type of platform."""
+    platform = await async_prepare_setup_platform(opp, config, DOMAIN, p_type)
+
+    if platform is None:
+        return None
+
+    return DeviceTrackerPlatform(p_type, platform, p_config)
+
+
+@callback
+def async_setup_scanner_platform(
+    opp: OpenPeerPowerType,
+    config: ConfigType,
+    scanner: Any,
+    async_see_device: Callable,
+    platform: str,
+):
+    """Set up the connect scanner-based platform to device tracker.
+
+    This method must be run in the event loop.
+    """
+    interval = config.get(CONF_SCAN_INTERVAL, SCAN_INTERVAL)
+    update_lock = asyncio.Lock()
+    scanner.opp = opp
+
+    # Initial scan of each mac we also tell about host name for config
+    seen: Any = set()
+
+    async def async_device_tracker_scan(now: dt_util.dt.datetime):
+        """Handle interval matches."""
+        if update_lock.locked():
+            LOGGER.warning(
+                "Updating device list from %s took longer than the scheduled "
+                "scan interval %s",
+                platform,
+                interval,
+            )
+            return
+
+        async with update_lock:
+            found_devices = await scanner.async_scan_devices()
+
+        for mac in found_devices:
+            if mac in seen:
+                host_name = None
+            else:
+                host_name = await scanner.async_get_device_name(mac)
+                seen.add(mac)
+
+            try:
+                extra_attributes = await scanner.async_get_extra_attributes(mac)
+            except NotImplementedError:
+                extra_attributes = {}
+
+            kwargs = {
+                "mac": mac,
+                "host_name": host_name,
+                "source_type": SOURCE_TYPE_ROUTER,
+                "attributes": {
+                    "scanner": scanner.__class__.__name__,
+                    **extra_attributes,
+                },
+            }
+
+            zone_home = opp.states.get(opp.components.zone.ENTITY_ID_HOME)
+            if zone_home:
+                kwargs["gps"] = [
+                    zone_home.attributes[ATTR_LATITUDE],
+                    zone_home.attributes[ATTR_LONGITUDE],
+                ]
+                kwargs["gps_accuracy"] = 0
+
+            opp.async_create_task(async_see_device(**kwargs))
+
+    async_track_time_interval(opp, async_device_tracker_scan, interval)
+    opp.async_create_task(async_device_tracker_scan(None))
+
+
 async def get_tracker(opp, config):
     """Create a tracker."""
     yaml_path = opp.config.path(YAML_DEVICES)
@@ -178,11 +486,11 @@ class DeviceTracker:
                 consider_home,
             )
             if device.track:
-                await device.async_update_op_state()
+                device.async_write_op_state()
             return
 
         # Guard from calling see on entity registry entities.
-        entity_id = ENTITY_ID_FORMAT.format(dev_id)
+        entity_id = f"{DOMAIN}.{dev_id}"
         if registry.async_is_registered(entity_id):
             LOGGER.error(
                 "The see service is not supported for this entity %s", entity_id
@@ -199,7 +507,6 @@ class DeviceTracker:
             mac,
             picture=picture,
             icon=icon,
-            hide_if_away=self.defaults.get(CONF_AWAY_HIDE, DEFAULT_AWAY_HIDE),
         )
         self.devices[dev_id] = device
         if mac is not None:
@@ -216,7 +523,7 @@ class DeviceTracker:
         )
 
         if device.track:
-            await device.async_update_op_state()
+            device.async_write_op_state()
 
         self.opp.bus.async_fire(
             EVENT_NEW_DEVICE,
@@ -229,9 +536,7 @@ class DeviceTracker:
 
         # update known_devices.yaml
         self.opp.async_create_task(
-            self.async_update_config(
-                self.opp.config.path(YAML_DEVICES), dev_id, device
-            )
+            self.async_update_config(self.opp.config.path(YAML_DEVICES), dev_id, device)
         )
 
     async def async_update_config(self, path, dev_id, device):
@@ -263,7 +568,7 @@ class DeviceTracker:
         async def async_init_single_device(dev):
             """Init a single device_tracker entity."""
             await dev.async_added_to_opp()
-            await dev.async_update_op_state()
+            dev.async_write_op_state()
 
         tasks = []
         for device in self.devices.values():
@@ -304,11 +609,10 @@ class Device(RestoreEntity):
         picture: str = None,
         gravatar: str = None,
         icon: str = None,
-        hide_if_away: bool = False,
     ) -> None:
         """Initialize a device."""
         self.opp = opp
-        self.entity_id = ENTITY_ID_FORMAT.format(dev_id)
+        self.entity_id = f"{DOMAIN}.{dev_id}"
 
         # Timedelta object how long we consider a device home if it is not
         # detected anymore.
@@ -332,8 +636,6 @@ class Device(RestoreEntity):
 
         self.icon = icon
 
-        self.away_hide = hide_if_away
-
         self.source_type = None
 
         self._attributes = {}
@@ -356,28 +658,23 @@ class Device(RestoreEntity):
     @property
     def state_attributes(self):
         """Return the device state attributes."""
-        attr = {ATTR_SOURCE_TYPE: self.source_type}
+        attributes = {ATTR_SOURCE_TYPE: self.source_type}
 
         if self.gps:
-            attr[ATTR_LATITUDE] = self.gps[0]
-            attr[ATTR_LONGITUDE] = self.gps[1]
-            attr[ATTR_GPS_ACCURACY] = self.gps_accuracy
+            attributes[ATTR_LATITUDE] = self.gps[0]
+            attributes[ATTR_LONGITUDE] = self.gps[1]
+            attributes[ATTR_GPS_ACCURACY] = self.gps_accuracy
 
         if self.battery:
-            attr[ATTR_BATTERY] = self.battery
+            attributes[ATTR_BATTERY] = self.battery
 
-        return attr
+        return attributes
 
     @property
     def device_state_attributes(self):
         """Return device state attributes."""
         return self._attributes
 
-    @property
-    def hidden(self):
-        """If device should be hidden."""
-        return self.away_hide and self.state != STATE_HOME
-
     async def async_seen(
         self,
         host_name: str = None,
@@ -412,7 +709,6 @@ class Device(RestoreEntity):
                 self.gps_accuracy = 0
                 LOGGER.warning("Could not parse gps value for %s: %s", self.dev_id, gps)
 
-        # pylint: disable=not-an-iterable
         await self.async_update()
 
     def stale(self, now: dt_util.dt.datetime = None):
@@ -466,13 +762,13 @@ class Device(RestoreEntity):
         self.last_update_home = state.state == STATE_HOME
         self.last_seen = dt_util.utcnow()
 
-        for attr, var in (
+        for attribute, var in (
             (ATTR_SOURCE_TYPE, "source_type"),
             (ATTR_GPS_ACCURACY, "gps_accuracy"),
             (ATTR_BATTERY, "battery"),
         ):
-            if attr in state.attributes:
-                setattr(self, var, state.attributes[attr])
+            if attribute in state.attributes:
+                setattr(self, var, state.attributes[attribute])
 
         if ATTR_LONGITUDE in state.attributes:
             self.gps = (
@@ -492,7 +788,7 @@ class DeviceScanner:
 
     async def async_scan_devices(self) -> Any:
         """Scan for devices."""
-        return await self.opp.async_add_job(self.scan_devices)
+        return await self.opp.async_add_executor_job(self.scan_devices)
 
     def get_device_name(self, device: str) -> str:
         """Get the name of a device."""
@@ -500,7 +796,7 @@ class DeviceScanner:
 
     async def async_get_device_name(self, device: str) -> Any:
         """Get the name of a device."""
-        return await self.opp.async_add_job(self.get_device_name, device)
+        return await self.opp.async_add_executor_job(self.get_device_name, device)
 
     def get_extra_attributes(self, device: str) -> dict:
         """Get the extra attributes of a device."""
@@ -508,7 +804,7 @@ class DeviceScanner:
 
     async def async_get_extra_attributes(self, device: str) -> Any:
         """Get the extra attributes of a device."""
-        return await self.opp.async_add_job(self.get_extra_attributes, device)
+        return await self.opp.async_add_executor_job(self.get_extra_attributes, device)
 
 
 async def async_load_config(
@@ -526,7 +822,6 @@ async def async_load_config(
             vol.Optional(CONF_MAC, default=None): vol.Any(
                 None, vol.All(cv.string, vol.Upper)
             ),
-            vol.Optional(CONF_AWAY_HIDE, default=DEFAULT_AWAY_HIDE): cv.boolean,
             vol.Optional("gravatar", default=None): vol.Any(None, cv.string),
             vol.Optional("picture", default=None): vol.Any(None, cv.string),
             vol.Optional(CONF_CONSIDER_HOME, default=consider_home): vol.All(
@@ -536,7 +831,7 @@ async def async_load_config(
     )
     result = []
     try:
-        devices = await opp.async_add_job(load_yaml_config_file, path)
+        devices = await opp.async_add_executor_job(load_yaml_config_file, path)
     except OpenPeerPowerError as err:
         LOGGER.error("Unable to load %s: %s", path, str(err))
         return []
@@ -546,6 +841,7 @@ async def async_load_config(
     for dev_id, device in devices.items():
         # Deprecated option. We just ignore it to avoid breaking change
         device.pop("vendor", None)
+        device.pop("hide_if_away", None)
         try:
             device = dev_schema(device)
             device["dev_id"] = cv.slugify(dev_id)
@@ -566,7 +862,6 @@ def update_config(path: str, dev_id: str, device: Device):
                 ATTR_ICON: device.icon,
                 "picture": device.config_picture,
                 "track": device.track,
-                CONF_AWAY_HIDE: device.away_hide,
             }
         }
         out.write("\n")
@@ -579,5 +874,7 @@ def get_gravatar_for_email(email: str):
     Async friendly.
     """
 
-    url = "https://www.gravatar.com/avatar/{}.jpg?s=80&d=wavatar"
-    return url.format(hashlib.md5(email.encode("utf-8").lower()).hexdigest())
+    return (
+        f"https://www.gravatar.com/avatar/"
+        f"{hashlib.md5(email.encode('utf-8').lower()).hexdigest()}.jpg?s=80&d=wavatar"
+    )
