@@ -4,7 +4,7 @@ import logging
 
 import voluptuous as vol
 
-from openpeerpower.components.climate import PLATFORM_SCHEMA, ClimateDevice
+from openpeerpower.components.climate import PLATFORM_SCHEMA, ClimateEntity
 from openpeerpower.components.climate.const import (
     ATTR_PRESET_MODE,
     CURRENT_HVAC_COOL,
@@ -23,6 +23,7 @@ from openpeerpower.const import (
     ATTR_ENTITY_ID,
     ATTR_TEMPERATURE,
     CONF_NAME,
+    CONF_UNIQUE_ID,
     EVENT_OPENPEERPOWER_START,
     PRECISION_HALVES,
     PRECISION_TENTHS,
@@ -30,17 +31,21 @@ from openpeerpower.const import (
     SERVICE_TURN_OFF,
     SERVICE_TURN_ON,
     STATE_ON,
+    STATE_UNAVAILABLE,
     STATE_UNKNOWN,
 )
-from openpeerpower.core import DOMAIN as HA_DOMAIN, callback
+from openpeerpower.core import DOMAIN as HA_DOMAIN, CoreState, callback
 from openpeerpower.helpers import condition
 import openpeerpower.helpers.config_validation as cv
 from openpeerpower.helpers.event import (
-    async_track_state_change,
+    async_track_state_change_event,
     async_track_time_interval,
 )
+from openpeerpower.helpers.reload import async_setup_reload_service
 from openpeerpower.helpers.restore_state import RestoreEntity
 
+from . import DOMAIN, PLATFORMS
+
 _LOGGER = logging.getLogger(__name__)
 
 DEFAULT_TOLERANCE = 0.3
@@ -67,13 +72,13 @@ PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
         vol.Required(CONF_SENSOR): cv.entity_id,
         vol.Optional(CONF_AC_MODE): cv.boolean,
         vol.Optional(CONF_MAX_TEMP): vol.Coerce(float),
-        vol.Optional(CONF_MIN_DUR): vol.All(cv.time_period, cv.positive_timedelta),
+        vol.Optional(CONF_MIN_DUR): cv.positive_time_period,
         vol.Optional(CONF_MIN_TEMP): vol.Coerce(float),
         vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
         vol.Optional(CONF_COLD_TOLERANCE, default=DEFAULT_TOLERANCE): vol.Coerce(float),
         vol.Optional(CONF_HOT_TOLERANCE, default=DEFAULT_TOLERANCE): vol.Coerce(float),
         vol.Optional(CONF_TARGET_TEMP): vol.Coerce(float),
-        vol.Optional(CONF_KEEP_ALIVE): vol.All(cv.time_period, cv.positive_timedelta),
+        vol.Optional(CONF_KEEP_ALIVE): cv.positive_time_period,
         vol.Optional(CONF_INITIAL_HVAC_MODE): vol.In(
             [HVAC_MODE_COOL, HVAC_MODE_HEAT, HVAC_MODE_OFF]
         ),
@@ -81,12 +86,16 @@ PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
         vol.Optional(CONF_PRECISION): vol.In(
             [PRECISION_TENTHS, PRECISION_HALVES, PRECISION_WHOLE]
         ),
+        vol.Optional(CONF_UNIQUE_ID): cv.string,
     }
 )
 
 
 async def async_setup_platform(opp, config, async_add_entities, discovery_info=None):
     """Set up the generic thermostat platform."""
+
+    await async_setup_reload_service(opp, DOMAIN, PLATFORMS)
+
     name = config.get(CONF_NAME)
     heater_entity_id = config.get(CONF_HEATER)
     sensor_entity_id = config.get(CONF_SENSOR)
@@ -102,6 +111,7 @@ async def async_setup_platform(opp, config, async_add_entities, discovery_info=N
     away_temp = config.get(CONF_AWAY_TEMP)
     precision = config.get(CONF_PRECISION)
     unit = opp.config.units.temperature_unit
+    unique_id = config.get(CONF_UNIQUE_ID)
 
     async_add_entities(
         [
@@ -121,12 +131,13 @@ async def async_setup_platform(opp, config, async_add_entities, discovery_info=N
                 away_temp,
                 precision,
                 unit,
+                unique_id,
             )
         ]
     )
 
 
-class GenericThermostat(ClimateDevice, RestoreEntity):
+class GenericThermostat(ClimateEntity, RestoreEntity):
     """Representation of a Generic Thermostat device."""
 
     def __init__(
@@ -146,6 +157,7 @@ class GenericThermostat(ClimateDevice, RestoreEntity):
         away_temp,
         precision,
         unit,
+        unique_id,
     ):
         """Initialize the thermostat."""
         self._name = name
@@ -170,6 +182,7 @@ class GenericThermostat(ClimateDevice, RestoreEntity):
         self._max_temp = max_temp
         self._target_temp = target_temp
         self._unit = unit
+        self._unique_id = unique_id
         self._support_flags = SUPPORT_FLAGS
         if away_temp:
             self._support_flags = SUPPORT_FLAGS | SUPPORT_PRESET_MODE
@@ -181,26 +194,39 @@ class GenericThermostat(ClimateDevice, RestoreEntity):
         await super().async_added_to_opp()
 
         # Add listener
-        async_track_state_change(
-            self.opp, self.sensor_entity_id, self._async_sensor_changed
+        self.async_on_remove(
+            async_track_state_change_event(
+                self.opp, [self.sensor_entity_id], self._async_sensor_changed
+            )
         )
-        async_track_state_change(
-            self.opp, self.heater_entity_id, self._async_switch_changed
+        self.async_on_remove(
+            async_track_state_change_event(
+                self.opp, [self.heater_entity_id], self._async_switch_changed
+            )
         )
 
         if self._keep_alive:
-            async_track_time_interval(
-                self.opp, self._async_control_heating, self._keep_alive
+            self.async_on_remove(
+                async_track_time_interval(
+                    self.opp, self._async_control_heating, self._keep_alive
+                )
             )
 
         @callback
-        def _async_startup(event):
+        def _async_startup(*_):
             """Init on startup."""
             sensor_state = self.opp.states.get(self.sensor_entity_id)
-            if sensor_state and sensor_state.state != STATE_UNKNOWN:
+            if sensor_state and sensor_state.state not in (
+                STATE_UNAVAILABLE,
+                STATE_UNKNOWN,
+            ):
                 self._async_update_temp(sensor_state)
+                self.async_write_op_state()
 
-        self.opp.bus.async_listen_once(EVENT_OPENPEERPOWER_START, _async_startup)
+        if self.opp.state == CoreState.running:
+            _async_startup()
+        else:
+            self.opp.bus.async_listen_once(EVENT_OPENPEERPOWER_START, _async_startup)
 
         # Check If we have an old state
         old_state = await self.async_get_last_state()
@@ -249,6 +275,11 @@ class GenericThermostat(ClimateDevice, RestoreEntity):
         """Return the name of the thermostat."""
         return self._name
 
+    @property
+    def unique_id(self):
+        """Return the unique id of this thermostat."""
+        return self._unique_id
+
     @property
     def precision(self):
         """Return the precision of the system."""
@@ -256,6 +287,13 @@ class GenericThermostat(ClimateDevice, RestoreEntity):
             return self._temp_precision
         return super().precision
 
+    @property
+    def target_temperature_step(self):
+        """Return the supported step of target temperature."""
+        # Since this integration does not yet have a step size parameter
+        # we have to re-use the precision as the step size for now.
+        return self.precision
+
     @property
     def temperature_unit(self):
         """Return the unit of measurement."""
@@ -321,7 +359,7 @@ class GenericThermostat(ClimateDevice, RestoreEntity):
             _LOGGER.error("Unrecognized hvac mode: %s", hvac_mode)
             return
         # Ensure we update the current operation after changing the mode
-        self.schedule_update_op_state()
+        self.async_write_op_state()
 
     async def async_set_temperature(self, **kwargs):
         """Set new target temperature."""
@@ -330,12 +368,12 @@ class GenericThermostat(ClimateDevice, RestoreEntity):
             return
         self._target_temp = temperature
         await self._async_control_heating(force=True)
-        await self.async_update_op_state()
+        self.async_write_op_state()
 
     @property
     def min_temp(self):
         """Return the minimum temperature."""
-        if self._min_temp:
+        if self._min_temp is not None:
             return self._min_temp
 
         # get default temp from super class
@@ -344,27 +382,29 @@ class GenericThermostat(ClimateDevice, RestoreEntity):
     @property
     def max_temp(self):
         """Return the maximum temperature."""
-        if self._max_temp:
+        if self._max_temp is not None:
             return self._max_temp
 
         # Get default temp from super class
         return super().max_temp
 
-    async def _async_sensor_changed(self, entity_id, old_state, new_state):
+    async def _async_sensor_changed(self, event):
         """Handle temperature changes."""
-        if new_state is None:
+        new_state = event.data.get("new_state")
+        if new_state is None or new_state.state in (STATE_UNAVAILABLE, STATE_UNKNOWN):
             return
 
         self._async_update_temp(new_state)
         await self._async_control_heating()
-        await self.async_update_op_state()
+        self.async_write_op_state()
 
     @callback
-    def _async_switch_changed(self, entity_id, old_state, new_state):
+    def _async_switch_changed(self, event):
         """Handle heater switch state changes."""
+        new_state = event.data.get("new_state")
         if new_state is None:
             return
-        self.async_schedule_update_op_state()
+        self.async_write_op_state()
 
     @callback
     def _async_update_temp(self, state):
@@ -445,12 +485,16 @@ class GenericThermostat(ClimateDevice, RestoreEntity):
     async def _async_heater_turn_on(self):
         """Turn heater toggleable device on."""
         data = {ATTR_ENTITY_ID: self.heater_entity_id}
-        await self.opp.services.async_call(HA_DOMAIN, SERVICE_TURN_ON, data)
+        await self.opp.services.async_call(
+            HA_DOMAIN, SERVICE_TURN_ON, data, context=self._context
+        )
 
     async def _async_heater_turn_off(self):
         """Turn heater toggleable device off."""
         data = {ATTR_ENTITY_ID: self.heater_entity_id}
-        await self.opp.services.async_call(HA_DOMAIN, SERVICE_TURN_OFF, data)
+        await self.opp.services.async_call(
+            HA_DOMAIN, SERVICE_TURN_OFF, data, context=self._context
+        )
 
     async def async_set_preset_mode(self, preset_mode: str):
         """Set new preset mode."""
@@ -464,4 +508,4 @@ class GenericThermostat(ClimateDevice, RestoreEntity):
             self._target_temp = self._saved_target_temp
             await self._async_control_heating(force=True)
 
-        await self.async_update_op_state()
+        self.async_write_op_state()
