@@ -1,4 +1,5 @@
 """Webhook handlers for mobile_app."""
+import asyncio
 from functools import wraps
 import logging
 import secrets
@@ -7,9 +8,11 @@ from aiohttp.web import HTTPBadRequest, Request, Response, json_response
 from nacl.secret import SecretBox
 import voluptuous as vol
 
+from openpeerpower.components import notify as opp_notify, tag
 from openpeerpower.components.binary_sensor import (
     DEVICE_CLASSES as BINARY_SENSOR_CLASSES,
 )
+from openpeerpower.components.camera import SUPPORT_STREAM as CAMERA_SUPPORT_STREAM
 from openpeerpower.components.device_tracker import (
     ATTR_BATTERY,
     ATTR_GPS,
@@ -20,18 +23,24 @@ from openpeerpower.components.frontend import MANIFEST_JSON
 from openpeerpower.components.sensor import DEVICE_CLASSES as SENSOR_CLASSES
 from openpeerpower.components.zone.const import DOMAIN as ZONE_DOMAIN
 from openpeerpower.const import (
+    ATTR_DEVICE_ID,
     ATTR_DOMAIN,
     ATTR_SERVICE,
     ATTR_SERVICE_DATA,
+    ATTR_SUPPORTED_FEATURES,
     CONF_WEBHOOK_ID,
     HTTP_BAD_REQUEST,
     HTTP_CREATED,
 )
 from openpeerpower.core import EventOrigin
-from openpeerpower.exceptions import OpenPeerPowerError, ServiceNotFound, TemplateError
-from openpeerpower.helpers import config_validation as cv, device_registry as dr
+from openpeerpower.exceptions import OpenPeerPowerError, ServiceNotFound
+from openpeerpower.helpers import (
+    config_validation as cv,
+    device_registry as dr,
+    entity_registry as er,
+    template,
+)
 from openpeerpower.helpers.dispatcher import async_dispatcher_send
-from openpeerpower.helpers.template import attach
 from openpeerpower.helpers.typing import OpenPeerPowerType
 from openpeerpower.util.decorator import Registry
 
@@ -39,8 +48,8 @@ from .const import (
     ATTR_ALTITUDE,
     ATTR_APP_DATA,
     ATTR_APP_VERSION,
+    ATTR_CAMERA_ENTITY_ID,
     ATTR_COURSE,
-    ATTR_DEVICE_ID,
     ATTR_DEVICE_NAME,
     ATTR_EVENT_DATA,
     ATTR_EVENT_TYPE,
@@ -71,12 +80,11 @@ from .const import (
     CONF_SECRET,
     DATA_CONFIG_ENTRIES,
     DATA_DELETED_IDS,
-    DATA_STORE,
     DOMAIN,
     ERR_ENCRYPTION_ALREADY_ENABLED,
     ERR_ENCRYPTION_NOT_AVAILABLE,
     ERR_ENCRYPTION_REQUIRED,
-    ERR_SENSOR_DUPLICATE_UNIQUE_ID,
+    ERR_INVALID_FORMAT,
     ERR_SENSOR_NOT_REGISTERED,
     SIGNAL_LOCATION_UPDATE,
     SIGNAL_SENSOR_UPDATE,
@@ -87,13 +95,13 @@ from .helpers import (
     error_response,
     registration_context,
     safe_registration,
-    savable_state,
     supports_encryption,
     webhook_response,
 )
 
 _LOGGER = logging.getLogger(__name__)
 
+DELAY_SAVE = 10
 
 WEBHOOK_COMMANDS = Registry()
 
@@ -144,10 +152,12 @@ async def handle_webhook(
 
     config_entry = opp.data[DOMAIN][DATA_CONFIG_ENTRIES][webhook_id]
 
+    device_name = config_entry.data[ATTR_DEVICE_NAME]
+
     try:
         req_data = await request.json()
     except ValueError:
-        _LOGGER.warning("Received invalid JSON from mobile_app")
+        _LOGGER.warning("Received invalid JSON from mobile_app device: %s", device_name)
         return empty_okay_response(status=HTTP_BAD_REQUEST)
 
     if (
@@ -156,7 +166,7 @@ async def handle_webhook(
     ):
         _LOGGER.warning(
             "Refusing to accept unencrypted webhook from %s",
-            config_entry.data[ATTR_DEVICE_NAME],
+            device_name,
         )
         return error_response(ERR_ENCRYPTION_REQUIRED, "Encryption required")
 
@@ -164,7 +174,9 @@ async def handle_webhook(
         req_data = WEBHOOK_PAYLOAD_SCHEMA(req_data)
     except vol.Invalid as ex:
         err = vol.humanize.humanize_error(req_data, ex)
-        _LOGGER.error("Received invalid webhook payload: %s", err)
+        _LOGGER.error(
+            "Received invalid webhook from %s with payload: %s", device_name, err
+        )
         return empty_okay_response()
 
     webhook_type = req_data[ATTR_WEBHOOK_TYPE]
@@ -176,14 +188,22 @@ async def handle_webhook(
         webhook_payload = _decrypt_payload(config_entry.data[CONF_SECRET], enc_data)
 
     if webhook_type not in WEBHOOK_COMMANDS:
-        _LOGGER.error("Received invalid webhook type: %s", webhook_type)
+        _LOGGER.error(
+            "Received invalid webhook from %s of type: %s", device_name, webhook_type
+        )
         return empty_okay_response()
 
     _LOGGER.debug(
-        "Received webhook payload for type %s: %s", webhook_type, webhook_payload
+        "Received webhook payload from %s for type %s: %s",
+        device_name,
+        webhook_type,
+        webhook_payload,
     )
 
-    return await WEBHOOK_COMMANDS[webhook_type](opp, config_entry, webhook_payload)
+    # Shield so we make sure we finish the webhook, even if sender hangs up.
+    return await asyncio.shield(
+        WEBHOOK_COMMANDS[webhook_type](opp, config_entry, webhook_payload)
+    )
 
 
 @WEBHOOK_COMMANDS.register("call_service")
@@ -211,7 +231,7 @@ async def webhook_call_service(opp, config_entry, data):
             config_entry.data[ATTR_DEVICE_NAME],
             ex,
         )
-        raise HTTPBadRequest()
+        raise HTTPBadRequest() from ex
 
     return empty_okay_response()
 
@@ -235,11 +255,39 @@ async def webhook_fire_event(opp, config_entry, data):
     return empty_okay_response()
 
 
+@WEBHOOK_COMMANDS.register("stream_camera")
+@validate_schema({vol.Required(ATTR_CAMERA_ENTITY_ID): cv.string})
+async def webhook_stream_camera(opp, config_entry, data):
+    """Handle a request to HLS-stream a camera."""
+    camera = opp.states.get(data[ATTR_CAMERA_ENTITY_ID])
+
+    if camera is None:
+        return webhook_response(
+            {"success": False},
+            registration=config_entry.data,
+            status=HTTP_BAD_REQUEST,
+        )
+
+    resp = {"mjpeg_path": "/api/camera_proxy_stream/%s" % (camera.entity_id)}
+
+    if camera.attributes[ATTR_SUPPORTED_FEATURES] & CAMERA_SUPPORT_STREAM:
+        try:
+            resp["hls_path"] = await opp.components.camera.async_request_stream(
+                camera.entity_id, "hls"
+            )
+        except OpenPeerPowerError:
+            resp["hls_path"] = None
+    else:
+        resp["hls_path"] = None
+
+    return webhook_response(resp, registration=config_entry.data)
+
+
 @WEBHOOK_COMMANDS.register("render_template")
 @validate_schema(
     {
         str: {
-            vol.Required(ATTR_TEMPLATE): cv.template,
+            vol.Required(ATTR_TEMPLATE): cv.string,
             vol.Optional(ATTR_TEMPLATE_VARIABLES, default={}): dict,
         }
     }
@@ -249,10 +297,9 @@ async def webhook_render_template(opp, config_entry, data):
     resp = {}
     for key, item in data.items():
         try:
-            tpl = item[ATTR_TEMPLATE]
-            attach(opp, tpl)
+            tpl = template.Template(item[ATTR_TEMPLATE], opp)
             resp[key] = tpl.async_render(item.get(ATTR_TEMPLATE_VARIABLES))
-        except TemplateError as ex:
+        except template.TemplateError as ex:
             resp[key] = {"error": str(ex)}
 
     return webhook_response(resp, registration=config_entry.data)
@@ -307,8 +354,11 @@ async def webhook_update_registration(opp, config_entry, data):
 
     opp.config_entries.async_update_entry(config_entry, data=new_registration)
 
+    await opp_notify.async_reload(opp, DOMAIN)
+
     return webhook_response(
-        safe_registration(new_registration), registration=new_registration,
+        safe_registration(new_registration),
+        registration=new_registration,
     )
 
 
@@ -351,41 +401,41 @@ async def webhook_enable_encryption(opp, config_entry, data):
         vol.Required(ATTR_SENSOR_TYPE): vol.In(SENSOR_TYPES),
         vol.Required(ATTR_SENSOR_UNIQUE_ID): cv.string,
         vol.Optional(ATTR_SENSOR_UOM): cv.string,
-        vol.Required(ATTR_SENSOR_STATE): vol.Any(bool, str, int, float),
+        vol.Optional(ATTR_SENSOR_STATE, default=None): vol.Any(
+            None, bool, str, int, float
+        ),
         vol.Optional(ATTR_SENSOR_ICON, default="mdi:cellphone"): cv.icon,
     }
 )
 async def webhook_register_sensor(opp, config_entry, data):
     """Handle a register sensor webhook."""
     entity_type = data[ATTR_SENSOR_TYPE]
-
     unique_id = data[ATTR_SENSOR_UNIQUE_ID]
+    device_name = config_entry.data[ATTR_DEVICE_NAME]
 
     unique_store_key = f"{config_entry.data[CONF_WEBHOOK_ID]}_{unique_id}"
-
-    if unique_store_key in opp.data[DOMAIN][entity_type]:
-        _LOGGER.error("Refusing to re-register existing sensor %s!", unique_id)
-        return error_response(
-            ERR_SENSOR_DUPLICATE_UNIQUE_ID,
-            f"{entity_type} {unique_id} already exists!",
-            status=409,
-        )
+    entity_registry = await er.async_get_registry(opp)
+    existing_sensor = entity_registry.async_get_entity_id(
+        entity_type, DOMAIN, unique_store_key
+    )
 
     data[CONF_WEBHOOK_ID] = config_entry.data[CONF_WEBHOOK_ID]
 
-    opp.data[DOMAIN][entity_type][unique_store_key] = data
+    # If sensor already is registered, update current state instead
+    if existing_sensor:
+        _LOGGER.debug(
+            "Re-register for %s of existing sensor %s", device_name, unique_id
+        )
 
-    try:
-        await opp.data[DOMAIN][DATA_STORE].async_save(savable_state(opp))
-    except OpenPeerPowerError as ex:
-        _LOGGER.error("Error registering sensor: %s", ex)
-        return empty_okay_response()
-
-    register_signal = "{}_{}_register".format(DOMAIN, data[ATTR_SENSOR_TYPE])
-    async_dispatcher_send(opp, register_signal, data)
+        async_dispatcher_send(opp, SIGNAL_SENSOR_UPDATE, data)
+    else:
+        register_signal = f"{DOMAIN}_{data[ATTR_SENSOR_TYPE]}_register"
+        async_dispatcher_send(opp, register_signal, data)
 
     return webhook_response(
-        {"success": True}, registration=config_entry.data, status=HTTP_CREATED,
+        {"success": True},
+        registration=config_entry.data,
+        status=HTTP_CREATED,
     )
 
 
@@ -394,20 +444,32 @@ async def webhook_register_sensor(opp, config_entry, data):
     vol.All(
         cv.ensure_list,
         [
+            # Partial schema, enough to identify schema.
+            # We don't validate everything because otherwise 1 invalid sensor
+            # will invalidate all sensors.
             vol.Schema(
                 {
-                    vol.Optional(ATTR_SENSOR_ATTRIBUTES, default={}): dict,
-                    vol.Optional(ATTR_SENSOR_ICON, default="mdi:cellphone"): cv.icon,
-                    vol.Required(ATTR_SENSOR_STATE): vol.Any(bool, str, int, float),
                     vol.Required(ATTR_SENSOR_TYPE): vol.In(SENSOR_TYPES),
                     vol.Required(ATTR_SENSOR_UNIQUE_ID): cv.string,
-                }
+                },
+                extra=vol.ALLOW_EXTRA,
             )
         ],
     )
 )
 async def webhook_update_sensor_states(opp, config_entry, data):
     """Handle an update sensor states webhook."""
+    sensor_schema_full = vol.Schema(
+        {
+            vol.Optional(ATTR_SENSOR_ATTRIBUTES, default={}): dict,
+            vol.Optional(ATTR_SENSOR_ICON, default="mdi:cellphone"): cv.icon,
+            vol.Required(ATTR_SENSOR_STATE): vol.Any(None, bool, str, int, float),
+            vol.Required(ATTR_SENSOR_TYPE): vol.In(SENSOR_TYPES),
+            vol.Required(ATTR_SENSOR_UNIQUE_ID): cv.string,
+        }
+    )
+
+    device_name = config_entry.data[ATTR_DEVICE_NAME]
     resp = {}
     for sensor in data:
         entity_type = sensor[ATTR_SENSOR_TYPE]
@@ -416,9 +478,14 @@ async def webhook_update_sensor_states(opp, config_entry, data):
 
         unique_store_key = f"{config_entry.data[CONF_WEBHOOK_ID]}_{unique_id}"
 
-        if unique_store_key not in opp.data[DOMAIN][entity_type]:
+        entity_registry = await er.async_get_registry(opp)
+        if not entity_registry.async_get_entity_id(
+            entity_type, DOMAIN, unique_store_key
+        ):
             _LOGGER.error(
-                "Refusing to update non-registered sensor: %s", unique_store_key
+                "Refusing to update %s non-registered sensor: %s",
+                device_name,
+                unique_store_key,
             )
             err_msg = f"{entity_type} {unique_id} is not registered"
             resp[unique_id] = {
@@ -427,19 +494,25 @@ async def webhook_update_sensor_states(opp, config_entry, data):
             }
             continue
 
-        entry = opp.data[DOMAIN][entity_type][unique_store_key]
-
-        new_state = {**entry, **sensor}
-
-        opp.data[DOMAIN][entity_type][unique_store_key] = new_state
-
-        safe = savable_state(opp)
+        entry = {CONF_WEBHOOK_ID: config_entry.data[CONF_WEBHOOK_ID]}
 
         try:
-            await opp.data[DOMAIN][DATA_STORE].async_save(safe)
-        except OpenPeerPowerError as ex:
-            _LOGGER.error("Error updating mobile_app registration: %s", ex)
-            return empty_okay_response()
+            sensor = sensor_schema_full(sensor)
+        except vol.Invalid as err:
+            err_msg = vol.humanize.humanize_error(sensor, err)
+            _LOGGER.error(
+                "Received invalid sensor payload from %s for %s: %s",
+                device_name,
+                unique_id,
+                err_msg,
+            )
+            resp[unique_id] = {
+                "success": False,
+                "error": {"code": ERR_INVALID_FORMAT, "message": err_msg},
+            }
+            continue
+
+        new_state = {**entry, **sensor}
 
         async_dispatcher_send(opp, SIGNAL_SENSOR_UPDATE, new_state)
 
@@ -484,3 +557,16 @@ async def webhook_get_config(opp, config_entry, data):
         pass
 
     return webhook_response(resp, registration=config_entry.data)
+
+
+@WEBHOOK_COMMANDS.register("scan_tag")
+@validate_schema({vol.Required("tag_id"): cv.string})
+async def webhook_scan_tag(opp, config_entry, data):
+    """Handle a fire event webhook."""
+    await tag.async_scan_tag(
+        opp,
+        data["tag_id"],
+        config_entry.data[ATTR_DEVICE_ID],
+        registration_context(config_entry.data),
+    )
+    return empty_okay_response()
