@@ -1,15 +1,9 @@
-"""
-Support for MQTT JSON lights.
-
-For more details about this platform, please refer to the documentation at
-https://open-peer-power.io/components/light.mqtt_json/
-"""
+"""Support for MQTT JSON lights."""
 import json
 import logging
 
 import voluptuous as vol
 
-from openpeerpower.components import mqtt
 from openpeerpower.components.light import (
     ATTR_BRIGHTNESS,
     ATTR_COLOR_TEMP,
@@ -27,28 +21,18 @@ from openpeerpower.components.light import (
     SUPPORT_FLASH,
     SUPPORT_TRANSITION,
     SUPPORT_WHITE_VALUE,
-    Light,
-)
-from openpeerpower.components.mqtt import (
-    CONF_COMMAND_TOPIC,
-    CONF_QOS,
-    CONF_RETAIN,
-    CONF_STATE_TOPIC,
-    CONF_UNIQUE_ID,
-    MqttAttributes,
-    MqttAvailability,
-    MqttDiscoveryUpdate,
-    MqttEntityDeviceInfo,
-    subscription,
+    LightEntity,
 )
 from openpeerpower.const import (
     CONF_BRIGHTNESS,
     CONF_COLOR_TEMP,
     CONF_DEVICE,
     CONF_EFFECT,
+    CONF_HS,
     CONF_NAME,
     CONF_OPTIMISTIC,
     CONF_RGB,
+    CONF_UNIQUE_ID,
     CONF_WHITE_VALUE,
     CONF_XY,
     STATE_ON,
@@ -59,6 +43,15 @@ from openpeerpower.helpers.restore_state import RestoreEntity
 from openpeerpower.helpers.typing import ConfigType
 import openpeerpower.util.color as color_util
 
+from .. import CONF_COMMAND_TOPIC, CONF_QOS, CONF_RETAIN, CONF_STATE_TOPIC, subscription
+from ... import mqtt
+from ..debug_info import log_messages
+from ..mixins import (
+    MQTT_AVAILABILITY_SCHEMA,
+    MQTT_ENTITY_DEVICE_INFO_SCHEMA,
+    MQTT_JSON_ATTRS_SCHEMA,
+    MqttEntity,
+)
 from .schema import MQTT_LIGHT_SCHEMA_SCHEMA
 from .schema_basic import CONF_BRIGHTNESS_SCALE
 
@@ -83,7 +76,9 @@ CONF_EFFECT_LIST = "effect_list"
 
 CONF_FLASH_TIME_LONG = "flash_time_long"
 CONF_FLASH_TIME_SHORT = "flash_time_short"
-CONF_HS = "hs"
+
+CONF_MAX_MIREDS = "max_mireds"
+CONF_MIN_MIREDS = "min_mireds"
 
 # Stealing some of these from the base MQTT configs.
 PLATFORM_SCHEMA_JSON = (
@@ -94,7 +89,7 @@ PLATFORM_SCHEMA_JSON = (
                 CONF_BRIGHTNESS_SCALE, default=DEFAULT_BRIGHTNESS_SCALE
             ): vol.All(vol.Coerce(int), vol.Range(min=1)),
             vol.Optional(CONF_COLOR_TEMP, default=DEFAULT_COLOR_TEMP): cv.boolean,
-            vol.Optional(CONF_DEVICE): mqtt.MQTT_ENTITY_DEVICE_INFO_SCHEMA,
+            vol.Optional(CONF_DEVICE): MQTT_ENTITY_DEVICE_INFO_SCHEMA,
             vol.Optional(CONF_EFFECT, default=DEFAULT_EFFECT): cv.boolean,
             vol.Optional(CONF_EFFECT_LIST): vol.All(cv.ensure_list, [cv.string]),
             vol.Optional(
@@ -104,6 +99,8 @@ PLATFORM_SCHEMA_JSON = (
                 CONF_FLASH_TIME_SHORT, default=DEFAULT_FLASH_TIME_SHORT
             ): cv.positive_int,
             vol.Optional(CONF_HS, default=DEFAULT_HS): cv.boolean,
+            vol.Optional(CONF_MAX_MIREDS): cv.positive_int,
+            vol.Optional(CONF_MIN_MIREDS): cv.positive_int,
             vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
             vol.Optional(CONF_OPTIMISTIC, default=DEFAULT_OPTIMISTIC): cv.boolean,
             vol.Optional(CONF_QOS, default=mqtt.DEFAULT_QOS): vol.All(
@@ -117,34 +114,25 @@ PLATFORM_SCHEMA_JSON = (
             vol.Optional(CONF_XY, default=DEFAULT_XY): cv.boolean,
         }
     )
-    .extend(mqtt.MQTT_AVAILABILITY_SCHEMA.schema)
-    .extend(mqtt.MQTT_JSON_ATTRS_SCHEMA.schema)
+    .extend(MQTT_AVAILABILITY_SCHEMA.schema)
+    .extend(MQTT_JSON_ATTRS_SCHEMA.schema)
     .extend(MQTT_LIGHT_SCHEMA_SCHEMA.schema)
 )
 
 
 async def async_setup_entity_json(
-    config: ConfigType, async_add_entities, config_entry, discovery_hash
+    opp, config: ConfigType, async_add_entities, config_entry, discovery_data
 ):
     """Set up a MQTT JSON Light."""
-    async_add_entities([MqttLightJson(config, config_entry, discovery_hash)])
+    async_add_entities([MqttLightJson(config, config_entry, discovery_data)])
 
 
-# pylint: disable=too-many-ancestors
-class MqttLightJson(
-    MqttAttributes,
-    MqttAvailability,
-    MqttDiscoveryUpdate,
-    MqttEntityDeviceInfo,
-    Light,
-    RestoreEntity,
-):
+class MqttLightJson(MqttEntity, LightEntity, RestoreEntity):
     """Representation of a MQTT JSON light."""
 
-    def __init__(self, config, config_entry, discovery_hash):
+    def __init__(self, config, config_entry, discovery_data):
         """Initialize MQTT JSON light."""
         self._state = False
-        self._sub_state = None
         self._supported_features = 0
 
         self._topic = None
@@ -155,32 +143,13 @@ class MqttLightJson(
         self._hs = None
         self._white_value = None
         self._flash_times = None
-        self._unique_id = config.get(CONF_UNIQUE_ID)
 
-        # Load config
-        self._setup_from_config(config)
+        MqttEntity.__init__(self, None, config, config_entry, discovery_data)
 
-        device_config = config.get(CONF_DEVICE)
-
-        MqttAttributes.__init__(self, config)
-        MqttAvailability.__init__(self, config)
-        MqttDiscoveryUpdate.__init__(self, discovery_hash, self.discovery_update)
-        MqttEntityDeviceInfo.__init__(self, device_config, config_entry)
-
-    async def async_added_to_opp(self):
-        """Subscribe to MQTT events."""
-        await super().async_added_to_opp()
-        await self._subscribe_topics()
-
-    async def discovery_update(self, discovery_payload):
-        """Handle updated discovery message."""
-        config = PLATFORM_SCHEMA_JSON(discovery_payload)
-        self._setup_from_config(config)
-        await self.attributes_discovery_update(config)
-        await self.availability_discovery_update(config)
-        await self.device_info_discovery_update(config)
-        await self._subscribe_topics()
-        self.async_write_op_state()
+    @staticmethod
+    def config_schema():
+        """Return the config schema."""
+        return PLATFORM_SCHEMA_JSON
 
     def _setup_from_config(self, config):
         """(Re)Setup the entity."""
@@ -192,35 +161,6 @@ class MqttLightJson(
         optimistic = config[CONF_OPTIMISTIC]
         self._optimistic = optimistic or self._topic[CONF_STATE_TOPIC] is None
 
-        brightness = config[CONF_BRIGHTNESS]
-        if brightness:
-            self._brightness = 255
-        else:
-            self._brightness = None
-
-        color_temp = config[CONF_COLOR_TEMP]
-        if color_temp:
-            self._color_temp = 150
-        else:
-            self._color_temp = None
-
-        effect = config[CONF_EFFECT]
-        if effect:
-            self._effect = "none"
-        else:
-            self._effect = None
-
-        white_value = config[CONF_WHITE_VALUE]
-        if white_value:
-            self._white_value = 255
-        else:
-            self._white_value = None
-
-        if config[CONF_HS] or config[CONF_RGB] or config[CONF_XY]:
-            self._hs = [0, 0]
-        else:
-            self._hs = None
-
         self._flash_times = {
             key: config.get(key)
             for key in (CONF_FLASH_TIME_SHORT, CONF_FLASH_TIME_LONG)
@@ -228,18 +168,56 @@ class MqttLightJson(
 
         self._supported_features = SUPPORT_TRANSITION | SUPPORT_FLASH
         self._supported_features |= config[CONF_RGB] and SUPPORT_COLOR
-        self._supported_features |= brightness and SUPPORT_BRIGHTNESS
-        self._supported_features |= color_temp and SUPPORT_COLOR_TEMP
-        self._supported_features |= effect and SUPPORT_EFFECT
-        self._supported_features |= white_value and SUPPORT_WHITE_VALUE
+        self._supported_features |= config[CONF_BRIGHTNESS] and SUPPORT_BRIGHTNESS
+        self._supported_features |= config[CONF_COLOR_TEMP] and SUPPORT_COLOR_TEMP
+        self._supported_features |= config[CONF_EFFECT] and SUPPORT_EFFECT
+        self._supported_features |= config[CONF_WHITE_VALUE] and SUPPORT_WHITE_VALUE
         self._supported_features |= config[CONF_XY] and SUPPORT_COLOR
         self._supported_features |= config[CONF_HS] and SUPPORT_COLOR
 
+    def _parse_color(self, values):
+        try:
+            red = int(values["color"]["r"])
+            green = int(values["color"]["g"])
+            blue = int(values["color"]["b"])
+
+            return color_util.color_RGB_to_hs(red, green, blue)
+        except KeyError:
+            pass
+        except ValueError:
+            _LOGGER.warning("Invalid RGB color value received")
+            return self._hs
+
+        try:
+            x_color = float(values["color"]["x"])
+            y_color = float(values["color"]["y"])
+
+            return color_util.color_xy_to_hs(x_color, y_color)
+        except KeyError:
+            pass
+        except ValueError:
+            _LOGGER.warning("Invalid XY color value received")
+            return self._hs
+
+        try:
+            hue = float(values["color"]["h"])
+            saturation = float(values["color"]["s"])
+
+            return (hue, saturation)
+        except KeyError:
+            pass
+        except ValueError:
+            _LOGGER.warning("Invalid HS color value received")
+            return self._hs
+
+        return self._hs
+
     async def _subscribe_topics(self):
         """(Re)Subscribe to topics."""
         last_state = await self.async_get_last_state()
 
         @callback
+        @log_messages(self.opp, self.entity_id)
         def state_received(msg):
             """Handle new MQTT messages."""
             values = json.loads(msg.payload)
@@ -249,39 +227,13 @@ class MqttLightJson(
             elif values["state"] == "OFF":
                 self._state = False
 
-            if self._hs is not None:
-                try:
-                    red = int(values["color"]["r"])
-                    green = int(values["color"]["g"])
-                    blue = int(values["color"]["b"])
+            if self._supported_features and SUPPORT_COLOR and "color" in values:
+                if values["color"] is None:
+                    self._hs = None
+                else:
+                    self._hs = self._parse_color(values)
 
-                    self._hs = color_util.color_RGB_to_hs(red, green, blue)
-                except KeyError:
-                    pass
-                except ValueError:
-                    _LOGGER.warning("Invalid RGB color value received")
-
-                try:
-                    x_color = float(values["color"]["x"])
-                    y_color = float(values["color"]["y"])
-
-                    self._hs = color_util.color_xy_to_hs(x_color, y_color)
-                except KeyError:
-                    pass
-                except ValueError:
-                    _LOGGER.warning("Invalid XY color value received")
-
-                try:
-                    hue = float(values["color"]["h"])
-                    saturation = float(values["color"]["s"])
-
-                    self._hs = (hue, saturation)
-                except KeyError:
-                    pass
-                except ValueError:
-                    _LOGGER.warning("Invalid HS color value received")
-
-            if self._brightness is not None:
+            if self._supported_features and SUPPORT_BRIGHTNESS:
                 try:
                     self._brightness = int(
                         values["brightness"]
@@ -290,26 +242,27 @@ class MqttLightJson(
                     )
                 except KeyError:
                     pass
-                except ValueError:
+                except (TypeError, ValueError):
                     _LOGGER.warning("Invalid brightness value received")
 
-            if self._color_temp is not None:
+            if self._supported_features and SUPPORT_COLOR_TEMP:
                 try:
-                    self._color_temp = int(values["color_temp"])
+                    if values["color_temp"] is None:
+                        self._color_temp = None
+                    else:
+                        self._color_temp = int(values["color_temp"])
                 except KeyError:
                     pass
                 except ValueError:
                     _LOGGER.warning("Invalid color temp value received")
 
-            if self._effect is not None:
+            if self._supported_features and SUPPORT_EFFECT:
                 try:
                     self._effect = values["effect"]
                 except KeyError:
                     pass
-                except ValueError:
-                    _LOGGER.warning("Invalid effect value received")
 
-            if self._white_value is not None:
+            if self._supported_features and SUPPORT_WHITE_VALUE:
                 try:
                     self._white_value = int(values["white_value"])
                 except KeyError:
@@ -345,14 +298,6 @@ class MqttLightJson(
             if last_state.attributes.get(ATTR_WHITE_VALUE):
                 self._white_value = last_state.attributes.get(ATTR_WHITE_VALUE)
 
-    async def async_will_remove_from_opp(self):
-        """Unsubscribe when removed."""
-        self._sub_state = await subscription.async_unsubscribe_topics(
-            self.opp, self._sub_state
-        )
-        await MqttAttributes.async_will_remove_from_opp(self)
-        await MqttAvailability.async_will_remove_from_opp(self)
-
     @property
     def brightness(self):
         """Return the brightness of this light between 0..255."""
@@ -363,6 +308,16 @@ class MqttLightJson(
         """Return the color temperature in mired."""
         return self._color_temp
 
+    @property
+    def min_mireds(self):
+        """Return the coldest color_temp that this light supports."""
+        return self._config.get(CONF_MIN_MIREDS, super().min_mireds)
+
+    @property
+    def max_mireds(self):
+        """Return the warmest color_temp that this light supports."""
+        return self._config.get(CONF_MAX_MIREDS, super().max_mireds)
+
     @property
     def effect(self):
         """Return the current effect."""
@@ -383,21 +338,11 @@ class MqttLightJson(
         """Return the white property."""
         return self._white_value
 
-    @property
-    def should_poll(self):
-        """No polling needed for a MQTT light."""
-        return False
-
     @property
     def name(self):
         """Return the name of the device if any."""
         return self._config[CONF_NAME]
 
-    @property
-    def unique_id(self):
-        """Return a unique ID."""
-        return self._unique_id
-
     @property
     def is_on(self):
         """Return true if device is on."""
@@ -430,12 +375,10 @@ class MqttLightJson(
             if self._config[CONF_RGB]:
                 # If there's a brightness topic set, we don't want to scale the
                 # RGB values given using the brightness.
-                if self._brightness is not None:
+                if self._config[CONF_BRIGHTNESS]:
                     brightness = 255
                 else:
-                    brightness = kwargs.get(
-                        ATTR_BRIGHTNESS, self._brightness if self._brightness else 255
-                    )
+                    brightness = kwargs.get(ATTR_BRIGHTNESS, 255)
                 rgb = color_util.color_hsv_to_RGB(
                     hs_color[0], hs_color[1], brightness / 255 * 100
                 )
@@ -465,12 +408,15 @@ class MqttLightJson(
         if ATTR_TRANSITION in kwargs:
             message["transition"] = kwargs[ATTR_TRANSITION]
 
-        if ATTR_BRIGHTNESS in kwargs and self._brightness is not None:
-            message["brightness"] = int(
-                kwargs[ATTR_BRIGHTNESS]
-                / float(DEFAULT_BRIGHTNESS_SCALE)
-                * self._config[CONF_BRIGHTNESS_SCALE]
+        if ATTR_BRIGHTNESS in kwargs and self._config[CONF_BRIGHTNESS]:
+            brightness_normalized = kwargs[ATTR_BRIGHTNESS] / DEFAULT_BRIGHTNESS_SCALE
+            brightness_scale = self._config[CONF_BRIGHTNESS_SCALE]
+            device_brightness = min(
+                round(brightness_normalized * brightness_scale), brightness_scale
             )
+            # Make sure the brightness is not rounded down to 0
+            device_brightness = max(device_brightness, 1)
+            message["brightness"] = device_brightness
 
             if self._optimistic:
                 self._brightness = kwargs[ATTR_BRIGHTNESS]
