@@ -1,18 +1,8 @@
 """Support for Legacy MQTT vacuum."""
 import json
-import logging
 
 import voluptuous as vol
 
-from openpeerpower.components import mqtt
-from openpeerpower.components.mqtt import (
-    CONF_UNIQUE_ID,
-    MqttAttributes,
-    MqttAvailability,
-    MqttDiscoveryUpdate,
-    MqttEntityDeviceInfo,
-    subscription,
-)
 from openpeerpower.components.vacuum import (
     SUPPORT_BATTERY,
     SUPPORT_CLEAN_SPOT,
@@ -25,17 +15,29 @@ from openpeerpower.components.vacuum import (
     SUPPORT_STOP,
     SUPPORT_TURN_OFF,
     SUPPORT_TURN_ON,
-    VacuumDevice,
+    VacuumEntity,
+)
+from openpeerpower.const import (
+    ATTR_SUPPORTED_FEATURES,
+    CONF_DEVICE,
+    CONF_NAME,
+    CONF_UNIQUE_ID,
 )
-from openpeerpower.const import ATTR_SUPPORTED_FEATURES, CONF_DEVICE, CONF_NAME
 from openpeerpower.core import callback
 import openpeerpower.helpers.config_validation as cv
 from openpeerpower.helpers.icon import icon_for_battery_level
 
+from .. import subscription
+from ... import mqtt
+from ..debug_info import log_messages
+from ..mixins import (
+    MQTT_AVAILABILITY_SCHEMA,
+    MQTT_ENTITY_DEVICE_INFO_SCHEMA,
+    MQTT_JSON_ATTRS_SCHEMA,
+    MqttEntity,
+)
 from .schema import MQTT_VACUUM_SCHEMA, services_to_strings, strings_to_services
 
-_LOGGER = logging.getLogger(__name__)
-
 SERVICE_TO_STRING = {
     SUPPORT_TURN_ON: "turn_on",
     SUPPORT_TURN_OFF: "turn_off",
@@ -115,7 +117,7 @@ PLATFORM_SCHEMA_LEGACY = (
             vol.Inclusive(CONF_CHARGING_TOPIC, "charging"): mqtt.valid_publish_topic,
             vol.Inclusive(CONF_CLEANING_TEMPLATE, "cleaning"): cv.template,
             vol.Inclusive(CONF_CLEANING_TOPIC, "cleaning"): mqtt.valid_publish_topic,
-            vol.Optional(CONF_DEVICE): mqtt.MQTT_ENTITY_DEVICE_INFO_SCHEMA,
+            vol.Optional(CONF_DEVICE): MQTT_ENTITY_DEVICE_INFO_SCHEMA,
             vol.Inclusive(CONF_DOCKED_TEMPLATE, "docked"): cv.template,
             vol.Inclusive(CONF_DOCKED_TOPIC, "docked"): mqtt.valid_publish_topic,
             vol.Inclusive(CONF_ERROR_TEMPLATE, "error"): cv.template,
@@ -155,30 +157,23 @@ PLATFORM_SCHEMA_LEGACY = (
             vol.Optional(mqtt.CONF_RETAIN, default=DEFAULT_RETAIN): cv.boolean,
         }
     )
-    .extend(mqtt.MQTT_AVAILABILITY_SCHEMA.schema)
-    .extend(mqtt.MQTT_JSON_ATTRS_SCHEMA.schema)
+    .extend(MQTT_AVAILABILITY_SCHEMA.schema)
+    .extend(MQTT_JSON_ATTRS_SCHEMA.schema)
     .extend(MQTT_VACUUM_SCHEMA.schema)
 )
 
 
 async def async_setup_entity_legacy(
-    config, async_add_entities, config_entry, discovery_hash
+    config, async_add_entities, config_entry, discovery_data
 ):
     """Set up a MQTT Vacuum Legacy."""
-    async_add_entities([MqttVacuum(config, config_entry, discovery_hash)])
+    async_add_entities([MqttVacuum(config, config_entry, discovery_data)])
 
 
-# pylint: disable=too-many-ancestors
-class MqttVacuum(
-    MqttAttributes,
-    MqttAvailability,
-    MqttDiscoveryUpdate,
-    MqttEntityDeviceInfo,
-    VacuumDevice,
-):
+class MqttVacuum(MqttEntity, VacuumEntity):
     """Representation of a MQTT-controlled legacy vacuum."""
 
-    def __init__(self, config, config_entry, discovery_info):
+    def __init__(self, config, config_entry, discovery_data):
         """Initialize the vacuum."""
         self._cleaning = False
         self._charging = False
@@ -188,18 +183,13 @@ class MqttVacuum(
         self._battery_level = 0
         self._fan_speed = "unknown"
         self._fan_speed_list = []
-        self._sub_state = None
-        self._unique_id = config.get(CONF_UNIQUE_ID)
 
-        # Load config
-        self._setup_from_config(config)
+        MqttEntity.__init__(self, None, config, config_entry, discovery_data)
 
-        device_config = config.get(CONF_DEVICE)
-
-        MqttAttributes.__init__(self, config)
-        MqttAvailability.__init__(self, config)
-        MqttDiscoveryUpdate.__init__(self, discovery_info, self.discovery_update)
-        MqttEntityDeviceInfo.__init__(self, device_config, config_entry)
+    @staticmethod
+    def config_schema():
+        """Return the config schema."""
+        return PLATFORM_SCHEMA_LEGACY
 
     def _setup_from_config(self, config):
         self._name = config[CONF_NAME]
@@ -250,27 +240,6 @@ class MqttVacuum(
             )
         }
 
-    async def discovery_update(self, discovery_payload):
-        """Handle updated discovery message."""
-        config = PLATFORM_SCHEMA_LEGACY(discovery_payload)
-        self._setup_from_config(config)
-        await self.attributes_discovery_update(config)
-        await self.availability_discovery_update(config)
-        await self.device_info_discovery_update(config)
-        await self._subscribe_topics()
-        self.async_write_op_state()
-
-    async def async_added_to_opp(self):
-        """Subscribe MQTT events."""
-        await super().async_added_to_opp()
-        await self._subscribe_topics()
-
-    async def async_will_remove_from_opp(self):
-        """Unsubscribe when removed."""
-        await subscription.async_unsubscribe_topics(self.opp, self._sub_state)
-        await MqttAttributes.async_will_remove_from_opp(self)
-        await MqttAvailability.async_will_remove_from_opp(self)
-
     async def _subscribe_topics(self):
         """(Re)Subscribe to topics."""
         for tpl in self._templates.values():
@@ -278,6 +247,7 @@ class MqttVacuum(
                 tpl.opp = self.opp
 
         @callback
+        @log_messages(self.opp, self.entity_id)
         def message_received(msg):
             """Handle new MQTT message."""
             if (
@@ -373,58 +343,37 @@ class MqttVacuum(
         """Return the name of the vacuum."""
         return self._name
 
-    @property
-    def should_poll(self):
-        """No polling needed for an MQTT vacuum."""
-        return False
-
     @property
     def is_on(self):
         """Return true if vacuum is on."""
         return self._cleaning
 
-    @property
-    def unique_id(self):
-        """Return a unique ID."""
-        return self._unique_id
-
     @property
     def status(self):
         """Return a status string for the vacuum."""
-        if self.supported_features & SUPPORT_STATUS == 0:
-            return None
-
         return self._status
 
     @property
     def fan_speed(self):
         """Return the status of the vacuum."""
-        if self.supported_features & SUPPORT_FAN_SPEED == 0:
-            return None
-
         return self._fan_speed
 
     @property
     def fan_speed_list(self):
         """Return the status of the vacuum."""
-        if self.supported_features & SUPPORT_FAN_SPEED == 0:
-            return []
         return self._fan_speed_list
 
     @property
     def battery_level(self):
         """Return the status of the vacuum."""
-        if self.supported_features & SUPPORT_BATTERY == 0:
-            return
-
         return max(0, min(100, self._battery_level))
 
     @property
     def battery_icon(self):
-        """Return the battery icon for the vacuum cleaner."""
-        if self.supported_features & SUPPORT_BATTERY == 0:
-            return
+        """Return the battery icon for the vacuum cleaner.
 
+        No need to check SUPPORT_BATTERY, this won't be called if battery_level is None.
+        """
         return icon_for_battery_level(
             battery_level=self.battery_level, charging=self._charging
         )
