@@ -1,4 +1,5 @@
 """Support for Z-Wave."""
+# pylint: disable=import-outside-toplevel
 import asyncio
 import copy
 from importlib import import_module
@@ -27,6 +28,7 @@ from openpeerpower.helpers.entity import generate_entity_id
 from openpeerpower.helpers.entity_component import DEFAULT_SCAN_INTERVAL
 from openpeerpower.helpers.entity_platform import EntityPlatform
 from openpeerpower.helpers.entity_registry import (
+    async_entries_for_config_entry,
     async_get_registry as async_get_entity_registry,
 )
 from openpeerpower.helpers.entity_values import EntityValues
@@ -34,7 +36,6 @@ from openpeerpower.helpers.event import async_track_time_change
 from openpeerpower.util import convert
 import openpeerpower.util.dt as dt_util
 
-from . import config_flow  # noqa: F401 pylint: disable=unused-import
 from . import const, websocket_api as wsapi, workaround
 from .const import (
     CONF_AUTOHEAL,
@@ -80,13 +81,15 @@ CONF_DEVICE_CONFIG = "device_config"
 CONF_DEVICE_CONFIG_GLOB = "device_config_glob"
 CONF_DEVICE_CONFIG_DOMAIN = "device_config_domain"
 
+DATA_ZWAVE_CONFIG_YAML_PRESENT = "zwave_config_yaml_present"
+
 DEFAULT_CONF_IGNORED = False
 DEFAULT_CONF_INVERT_OPENCLOSE_BUTTONS = False
 DEFAULT_CONF_INVERT_PERCENT = False
 DEFAULT_CONF_REFRESH_VALUE = False
 DEFAULT_CONF_REFRESH_DELAY = 5
 
-SUPPORTED_PLATFORMS = [
+PLATFORMS = [
     "binary_sensor",
     "climate",
     "cover",
@@ -126,8 +129,8 @@ SET_CONFIG_PARAMETER_SCHEMA = vol.Schema(
 SET_NODE_VALUE_SCHEMA = vol.Schema(
     {
         vol.Required(const.ATTR_NODE_ID): vol.Coerce(int),
-        vol.Required(const.ATTR_VALUE_ID): vol.Coerce(int),
-        vol.Required(const.ATTR_CONFIG_VALUE): vol.Coerce(int),
+        vol.Required(const.ATTR_VALUE_ID): vol.Any(vol.Coerce(int), cv.string),
+        vol.Required(const.ATTR_CONFIG_VALUE): vol.Any(vol.Coerce(int), cv.string),
     }
 )
 
@@ -249,6 +252,64 @@ CONFIG_SCHEMA = vol.Schema(
 )
 
 
+async def async_get_ozw_migration_data(opp):
+    """Return dict with info for migration to ozw integration."""
+    data_to_migrate = {}
+
+    zwave_config_entries = opp.config_entries.async_entries(DOMAIN)
+    if not zwave_config_entries:
+        _LOGGER.error("Config entry not set up")
+        return data_to_migrate
+
+    if opp.data.get(DATA_ZWAVE_CONFIG_YAML_PRESENT):
+        _LOGGER.warning(
+            "Remove %s from configuration.yaml "
+            "to avoid setting up this integration on restart "
+            "after completing migration to ozw",
+            DOMAIN,
+        )
+
+    config_entry = zwave_config_entries[0]  # zwave only has a single config entry
+    ent_reg = await async_get_entity_registry(opp)
+    entity_entries = async_entries_for_config_entry(ent_reg, config_entry.entry_id)
+    unique_entries = {entry.unique_id: entry for entry in entity_entries}
+    dev_reg = await async_get_device_registry(opp)
+
+    for entity_values in opp.data[DATA_ENTITY_VALUES]:
+        node = entity_values.primary.node
+        unique_id = compute_value_unique_id(node, entity_values.primary)
+        if unique_id not in unique_entries:
+            continue
+        device_identifier, _ = node_device_id_and_name(
+            node, entity_values.primary.instance
+        )
+        device_entry = dev_reg.async_get_device({device_identifier}, set())
+        data_to_migrate[unique_id] = {
+            "node_id": node.node_id,
+            "node_instance": entity_values.primary.instance,
+            "device_id": device_entry.id,
+            "command_class": entity_values.primary.command_class,
+            "command_class_label": entity_values.primary.label,
+            "value_index": entity_values.primary.index,
+            "unique_id": unique_id,
+            "entity_entry": unique_entries[unique_id],
+        }
+
+    return data_to_migrate
+
+
+@callback
+def async_is_ozw_migrated(opp):
+    """Return True if migration to ozw is done."""
+    ozw_config_entries = opp.config_entries.async_entries("ozw")
+    if not ozw_config_entries:
+        return False
+
+    ozw_config_entry = ozw_config_entries[0]  # only one ozw entry is allowed
+    migrated = bool(ozw_config_entry.data.get("migrated"))
+    return migrated
+
+
 def _obj_to_dict(obj):
     """Convert an object into a hash for debug."""
     return {
@@ -260,7 +321,7 @@ def _obj_to_dict(obj):
 
 def _value_name(value):
     """Return the name of the value."""
-    return "{} {}".format(node_name(value.node), value.label).strip()
+    return f"{node_name(value.node)} {value.label}".strip()
 
 
 def nice_print_node(node):
@@ -296,7 +357,7 @@ async def async_setup_platform(opp, config, async_add_entities, discovery_info=N
     if discovery_info is None or DATA_NETWORK not in opp.data:
         return False
 
-    device = opp.data[DATA_DEVICES].get(discovery_info[const.DISCOVERY_DEVICE], None)
+    device = opp.data[DATA_DEVICES].get(discovery_info[const.DISCOVERY_DEVICE])
     if device is None:
         return False
 
@@ -311,6 +372,7 @@ async def async_setup(opp, config):
 
     conf = config[DOMAIN]
     opp.data[DATA_ZWAVE_CONFIG] = conf
+    opp.data[DATA_ZWAVE_CONFIG_YAML_PRESENT] = True
 
     if not opp.config_entries.async_entries(DOMAIN):
         opp.async_create_task(
@@ -334,12 +396,18 @@ async def async_setup_entry(opp, config_entry):
 
     Will automatically load components to support devices found on the network.
     """
+    from openzwave.group import ZWaveGroup
+    from openzwave.network import ZWaveNetwork
+    from openzwave.option import ZWaveOption
+
+    # pylint: enable=import-error
     from pydispatch import dispatcher
 
-    # pylint: disable=import-error
-    from openzwave.option import ZWaveOption
-    from openzwave.network import ZWaveNetwork
-    from openzwave.group import ZWaveGroup
+    if async_is_ozw_migrated(opp):
+        _LOGGER.error(
+            "Migration to ozw has been done. Please remove the zwave integration"
+        )
+        return False
 
     # Merge config entry and yaml config
     config = config_entry.data
@@ -503,7 +571,7 @@ async def async_setup_entry(opp, config_entry):
     async def _remove_device(node):
         dev_reg = await async_get_device_registry(opp)
         identifier, name = node_device_id_and_name(node)
-        device = dev_reg.async_get_device(identifiers={identifier}, connections=set())
+        device = dev_reg.async_get_device(identifiers={identifier})
         if device is not None:
             _LOGGER.debug("Removing Device - %s - %s", device.id, name)
             dev_reg.async_remove_device(device.id)
@@ -513,7 +581,7 @@ async def async_setup_entry(opp, config_entry):
         _LOGGER.info(
             "Z-Wave network is ready for use. All awake nodes "
             "have been queried. Sleeping nodes will be "
-            "queried when they awake."
+            "queried when they awake"
         )
         opp.bus.fire(const.EVENT_NETWORK_READY)
 
@@ -592,7 +660,7 @@ async def async_setup_entry(opp, config_entry):
     async def rename_node(service):
         """Rename a node."""
         node_id = service.data.get(const.ATTR_NODE_ID)
-        node = network.nodes[node_id]
+        node = network.nodes[node_id]  # pylint: disable=unsubscriptable-object
         name = service.data.get(const.ATTR_NAME)
         node.name = name
         _LOGGER.info("Renamed Z-Wave node %d to %s", node_id, name)
@@ -612,7 +680,7 @@ async def async_setup_entry(opp, config_entry):
         """Rename a node value."""
         node_id = service.data.get(const.ATTR_NODE_ID)
         value_id = service.data.get(const.ATTR_VALUE_ID)
-        node = network.nodes[node_id]
+        node = network.nodes[node_id]  # pylint: disable=unsubscriptable-object
         value = node.values[value_id]
         name = service.data.get(const.ATTR_NAME)
         value.label = name
@@ -628,7 +696,7 @@ async def async_setup_entry(opp, config_entry):
         """Set the polling intensity of a node value."""
         node_id = service.data.get(const.ATTR_NODE_ID)
         value_id = service.data.get(const.ATTR_VALUE_ID)
-        node = network.nodes[node_id]
+        node = network.nodes[node_id]  # pylint: disable=unsubscriptable-object
         value = node.values[value_id]
         intensity = service.data.get(const.ATTR_POLL_INTENSITY)
         if intensity == 0:
@@ -666,7 +734,7 @@ async def async_setup_entry(opp, config_entry):
     def set_config_parameter(service):
         """Set a config parameter to a node."""
         node_id = service.data.get(const.ATTR_NODE_ID)
-        node = network.nodes[node_id]
+        node = network.nodes[node_id]  # pylint: disable=unsubscriptable-object
         param = service.data.get(const.ATTR_CONFIG_PARAMETER)
         selection = service.data.get(const.ATTR_CONFIG_VALUE)
         size = service.data.get(const.ATTR_CONFIG_SIZE)
@@ -724,7 +792,7 @@ async def async_setup_entry(opp, config_entry):
         """Refresh the specified value from a node."""
         node_id = service.data.get(const.ATTR_NODE_ID)
         value_id = service.data.get(const.ATTR_VALUE_ID)
-        node = network.nodes[node_id]
+        node = network.nodes[node_id]  # pylint: disable=unsubscriptable-object
         node.values[value_id].refresh()
         _LOGGER.info("Node %s value %s refreshed", node_id, value_id)
 
@@ -733,14 +801,14 @@ async def async_setup_entry(opp, config_entry):
         node_id = service.data.get(const.ATTR_NODE_ID)
         value_id = service.data.get(const.ATTR_VALUE_ID)
         value = service.data.get(const.ATTR_CONFIG_VALUE)
-        node = network.nodes[node_id]
+        node = network.nodes[node_id]  # pylint: disable=unsubscriptable-object
         node.values[value_id].data = value
         _LOGGER.info("Node %s value %s set to %s", node_id, value_id, value)
 
     def print_config_parameter(service):
         """Print a config parameter from a node."""
         node_id = service.data.get(const.ATTR_NODE_ID)
-        node = network.nodes[node_id]
+        node = network.nodes[node_id]  # pylint: disable=unsubscriptable-object
         param = service.data.get(const.ATTR_CONFIG_PARAMETER)
         _LOGGER.info(
             "Config parameter %s on Node %s: %s",
@@ -752,13 +820,13 @@ async def async_setup_entry(opp, config_entry):
     def print_node(service):
         """Print all information about z-wave node."""
         node_id = service.data.get(const.ATTR_NODE_ID)
-        node = network.nodes[node_id]
+        node = network.nodes[node_id]  # pylint: disable=unsubscriptable-object
         nice_print_node(node)
 
     def set_wakeup(service):
         """Set wake-up interval of a node."""
         node_id = service.data.get(const.ATTR_NODE_ID)
-        node = network.nodes[node_id]
+        node = network.nodes[node_id]  # pylint: disable=unsubscriptable-object
         value = service.data.get(const.ATTR_CONFIG_VALUE)
         if node.can_wake_up():
             for value_id in node.get_values(class_id=const.COMMAND_CLASS_WAKE_UP):
@@ -805,14 +873,14 @@ async def async_setup_entry(opp, config_entry):
     def refresh_node(service):
         """Refresh all node info."""
         node_id = service.data.get(const.ATTR_NODE_ID)
-        node = network.nodes[node_id]
+        node = network.nodes[node_id]  # pylint: disable=unsubscriptable-object
         node.refresh_info()
 
     def reset_node_meters(service):
         """Reset meter counters of a node."""
         node_id = service.data.get(const.ATTR_NODE_ID)
         instance = service.data.get(const.ATTR_INSTANCE)
-        node = network.nodes[node_id]
+        node = network.nodes[node_id]  # pylint: disable=unsubscriptable-object
         for value in node.get_values(class_id=const.COMMAND_CLASS_METER).values():
             if value.index != const.INDEX_METER_RESET:
                 continue
@@ -825,16 +893,14 @@ async def async_setup_entry(opp, config_entry):
             )
             return
         _LOGGER.info(
-            "Node %s on instance %s does not have resettable meters.",
-            node_id,
-            instance,
+            "Node %s on instance %s does not have resettable meters", node_id, instance
         )
 
     def heal_node(service):
         """Heal a node on the network."""
         node_id = service.data.get(const.ATTR_NODE_ID)
         update_return_routes = service.data.get(const.ATTR_RETURN_ROUTES)
-        node = network.nodes[node_id]
+        node = network.nodes[node_id]  # pylint: disable=unsubscriptable-object
         _LOGGER.info("Z-Wave node heal running for node %s", node_id)
         node.heal(update_return_routes)
 
@@ -842,8 +908,8 @@ async def async_setup_entry(opp, config_entry):
         """Send test messages to a node on the network."""
         node_id = service.data.get(const.ATTR_NODE_ID)
         messages = service.data.get(const.ATTR_MESSAGES)
-        node = network.nodes[node_id]
-        _LOGGER.info("Sending %s test-messages to node %s.", messages, node_id)
+        node = network.nodes[node_id]  # pylint: disable=unsubscriptable-object
+        _LOGGER.info("Sending %s test-messages to node %s", messages, node_id)
         node.test(messages)
 
     def start_zwave(_service_or_event):
@@ -994,7 +1060,7 @@ async def async_setup_entry(opp, config_entry):
 
     opp.services.async_register(DOMAIN, const.SERVICE_START_NETWORK, start_zwave)
 
-    for entry_component in SUPPORTED_PLATFORMS:
+    for entry_component in PLATFORMS:
         opp.async_create_task(
             opp.config_entries.async_forward_entry_setup(config_entry, entry_component)
         )
@@ -1080,7 +1146,7 @@ class ZWaveDeviceEntityValues:
         if workaround_component and workaround_component != component:
             if workaround_component == workaround.WORKAROUND_IGNORE:
                 _LOGGER.info(
-                    "Ignoring Node %d Value %d due to workaround.",
+                    "Ignoring Node %d Value %d due to workaround",
                     self.primary.node.node_id,
                     self.primary.value_id,
                 )
@@ -1162,7 +1228,7 @@ class ZWaveDeviceEntityValues:
                 return
 
             self._opp.data[DATA_DEVICES][device.unique_id] = device
-            if component in SUPPORTED_PLATFORMS:
+            if component in PLATFORMS:
                 async_dispatcher_send(self._opp, f"zwave_new_{component}", device)
             else:
                 await discovery.async_load_platform(
@@ -1184,7 +1250,6 @@ class ZWaveDeviceEntity(ZWaveBaseEntity):
 
     def __init__(self, values, domain):
         """Initialize the z-Wave device."""
-        # pylint: disable=import-error
         super().__init__()
         from openzwave.network import ZWaveNetwork
         from pydispatch import dispatcher
@@ -1208,7 +1273,6 @@ class ZWaveDeviceEntity(ZWaveBaseEntity):
 
     def value_added(self):
         """Handle a new value of this entity."""
-        pass
 
     def value_changed(self):
         """Handle a changed value for this entity's node."""
@@ -1234,7 +1298,7 @@ class ZWaveDeviceEntity(ZWaveBaseEntity):
                 ent_reg.async_update_entity(self.entity_id, new_entity_id=new_entity_id)
                 return
         # else for the above two ifs, update if not using update_entity
-        self.async_schedule_update_op_state()
+        self.async_write_op_state()
 
     async def async_added_to_opp(self):
         """Add device to dict."""
@@ -1262,7 +1326,6 @@ class ZWaveDeviceEntity(ZWaveBaseEntity):
 
     def update_properties(self):
         """Update on data changes for node values."""
-        pass
 
     @property
     def should_poll(self):
