@@ -1,23 +1,42 @@
 """Class to manage the entities for a single platform."""
+from __future__ import annotations
+
 import asyncio
 from contextvars import ContextVar
-from datetime import datetime
-from typing import Optional
+from datetime import datetime, timedelta
+from logging import Logger
+from types import ModuleType
+from typing import TYPE_CHECKING, Callable, Coroutine, Dict, Iterable, List, Optional
 
-from openpeerpower.const import DEVICE_DEFAULT_NAME
-from openpeerpower.core import callback, split_entity_id, valid_entity_id
+from openpeerpower import config_entries
+from openpeerpower.const import ATTR_RESTORED, DEVICE_DEFAULT_NAME
+from openpeerpower.core import (
+    CALLBACK_TYPE,
+    ServiceCall,
+    callback,
+    split_entity_id,
+    valid_entity_id,
+)
 from openpeerpower.exceptions import OpenPeerPowerError, PlatformNotReady
 from openpeerpower.helpers import config_validation as cv, service
+from openpeerpower.helpers.typing import OpenPeerPowerType
 from openpeerpower.util.async_ import run_callback_threadsafe
 
 from .entity_registry import DISABLED_INTEGRATION
 from .event import async_call_later, async_track_time_interval
 
-# mypy: allow-untyped-defs, no-check-untyped-defs
+if TYPE_CHECKING:
+    from .entity import Entity
+
 
 SLOW_SETUP_WARNING = 10
 SLOW_SETUP_MAX_WAIT = 60
+SLOW_ADD_ENTITY_MAX_WAIT = 15  # Per Entity
+SLOW_ADD_MIN_TIMEOUT = 500
+
 PLATFORM_NOT_READY_RETRIES = 10
+DATA_ENTITY_PLATFORM = "entity_platform"
+PLATFORM_NOT_READY_BASE_WAIT_TIME = 30  # seconds
 
 
 class EntityPlatform:
@@ -26,23 +45,15 @@ class EntityPlatform:
     def __init__(
         self,
         *,
-        opp,
-        logger,
-        domain,
-        platform_name,
-        platform,
-        scan_interval,
-        entity_namespace,
+        opp: OpenPeerPowerType,
+        logger: Logger,
+        domain: str,
+        platform_name: str,
+        platform: Optional[ModuleType],
+        scan_interval: timedelta,
+        entity_namespace: Optional[str],
     ):
-        """Initialize the entity platform.
-
-        opp: OpenPeerPower
-        logger: Logger
-        domain: str
-        platform_name: str
-        scan_interval: timedelta
-        entity_namespace: str
-        """
+        """Initialize the entity platform."""
         self.opp = opp
         self.logger = logger
         self.domain = domain
@@ -50,24 +61,30 @@ class EntityPlatform:
         self.platform = platform
         self.scan_interval = scan_interval
         self.entity_namespace = entity_namespace
-        self.config_entry = None
-        self.entities = {}
-        self._tasks = []
+        self.config_entry: Optional[config_entries.ConfigEntry] = None
+        self.entities: Dict[str, Entity] = {}
+        self._tasks: List[asyncio.Future] = []
+        # Stop tracking tasks after setup is completed
+        self._setup_complete = False
         # Method to cancel the state change listener
-        self._async_unsub_polling = None
+        self._async_unsub_polling: Optional[CALLBACK_TYPE] = None
         # Method to cancel the retry of setup
-        self._async_cancel_retry_setup = None
-        self._process_updates = None
+        self._async_cancel_retry_setup: Optional[CALLBACK_TYPE] = None
+        self._process_updates: Optional[asyncio.Lock] = None
+
+        self.parallel_updates: Optional[asyncio.Semaphore] = None
 
         # Platform is None for the EntityComponent "catch-all" EntityPlatform
         # which powers entity_component.add_entities
-        if platform is None:
-            self.parallel_updates_created = True
-            self.parallel_updates: Optional[asyncio.Semaphore] = None
-            return
+        self.parallel_updates_created = platform is None
 
-        self.parallel_updates_created = False
-        self.parallel_updates = None
+        opp.data.setdefault(DATA_ENTITY_PLATFORM, {}).setdefault(
+            self.platform_name, []
+        ).append(self)
+
+    def __repr__(self) -> str:
+        """Represent an EntityPlatform."""
+        return f"<EntityPlatform domain={self.domain} platform_name={self.platform_name} config_entry={self.config_entry}>"
 
     @callback
     def _get_parallel_updates_semaphore(
@@ -79,7 +96,8 @@ class EntityPlatform:
 
         If parallel updates is set to 0, we skip the semaphore.
         If parallel updates is set to a number, we initialize the semaphore to that number.
-        Default for entities with `async_update` method is 1. Otherwise it's 0.
+        The default value for parallel requests is decided based on the first entity that is added to Open Peer Power.
+        It's 0 if the entity defines the async_update method, else it's 1.
         """
         if self.parallel_updates_created:
             return self.parallel_updates
@@ -99,7 +117,7 @@ class EntityPlatform:
 
         return self.parallel_updates
 
-    async def async_setup(self, platform_config, discovery_info=None):
+    async def async_setup(self, platform_config, discovery_info=None):  # type: ignore[no-untyped-def]
         """Set up the platform from a config file."""
         platform = self.platform
         opp = self.opp
@@ -115,10 +133,10 @@ class EntityPlatform:
             return
 
         @callback
-        def async_create_setup_task():
+        def async_create_setup_task() -> Coroutine:
             """Get task to set up platform."""
             if getattr(platform, "async_setup_platform", None):
-                return platform.async_setup_platform(
+                return platform.async_setup_platform(  # type: ignore
                     opp,
                     platform_config,
                     self._async_schedule_add_entities,
@@ -127,9 +145,9 @@ class EntityPlatform:
 
             # This should not be replaced with opp.async_add_job because
             # we don't want to track this task in case it blocks startup.
-            return opp.loop.run_in_executor(
+            return opp.loop.run_in_executor(  # type: ignore[return-value]
                 None,
-                platform.setup_platform,
+                platform.setup_platform,  # type: ignore
                 opp,
                 platform_config,
                 self._schedule_add_entities,
@@ -138,22 +156,24 @@ class EntityPlatform:
 
         await self._async_setup_platform(async_create_setup_task)
 
-    async def async_setup_entry(self, config_entry):
+    async def async_setup_entry(self, config_entry: config_entries.ConfigEntry) -> bool:
         """Set up the platform from a config entry."""
         # Store it so that we can save config entry ID in entity registry
         self.config_entry = config_entry
         platform = self.platform
 
         @callback
-        def async_create_setup_task():
+        def async_create_setup_task():  # type: ignore[no-untyped-def]
             """Get task to set up platform."""
-            return platform.async_setup_entry(
+            return platform.async_setup_entry(  # type: ignore
                 self.opp, config_entry, self._async_schedule_add_entities
             )
 
         return await self._async_setup_platform(async_create_setup_task)
 
-    async def _async_setup_platform(self, async_create_setup_task, tries=0):
+    async def _async_setup_platform(
+        self, async_create_setup_task: Callable[[], Coroutine], tries: int = 0
+    ) -> bool:
         """Set up a platform via config file or config entry.
 
         async_create_setup_task creates a coroutine that sets up platform.
@@ -176,28 +196,30 @@ class EntityPlatform:
         try:
             task = async_create_setup_task()
 
-            await asyncio.wait_for(asyncio.shield(task), SLOW_SETUP_MAX_WAIT)
+            async with opp.timeout.async_timeout(SLOW_SETUP_MAX_WAIT, self.domain):
+                await asyncio.shield(task)
 
             # Block till all entities are done
-            if self._tasks:
+            while self._tasks:
                 pending = [task for task in self._tasks if not task.done()]
                 self._tasks.clear()
 
                 if pending:
-                    await asyncio.wait(pending)
+                    await asyncio.gather(*pending)
 
             opp.config.components.add(full_name)
+            self._setup_complete = True
             return True
         except PlatformNotReady:
             tries += 1
-            wait_time = min(tries, 6) * 30
+            wait_time = min(tries, 6) * PLATFORM_NOT_READY_BASE_WAIT_TIME
             logger.warning(
                 "Platform %s not ready yet. Retrying in %d seconds.",
                 self.platform_name,
                 wait_time,
             )
 
-            async def setup_again(now):
+            async def setup_again(now):  # type: ignore[no-untyped-def]
                 """Run setup again."""
                 self._async_cancel_retry_setup = None
                 await self._async_setup_platform(async_create_setup_task, tries)
@@ -224,7 +246,9 @@ class EntityPlatform:
         finally:
             warn_task.cancel()
 
-    def _schedule_add_entities(self, new_entities, update_before_add=False):
+    def _schedule_add_entities(
+        self, new_entities: Iterable[Entity], update_before_add: bool = False
+    ) -> None:
         """Schedule adding entities for a single platform, synchronously."""
         run_callback_threadsafe(
             self.opp.loop,
@@ -234,17 +258,20 @@ class EntityPlatform:
         ).result()
 
     @callback
-    def _async_schedule_add_entities(self, new_entities, update_before_add=False):
+    def _async_schedule_add_entities(
+        self, new_entities: Iterable[Entity], update_before_add: bool = False
+    ) -> None:
         """Schedule adding entities for a single platform async."""
-        self._tasks.append(
-            self.opp.async_add_job(
-                self.async_add_entities(
-                    new_entities, update_before_add=update_before_add
-                )
-            )
+        task = self.opp.async_create_task(
+            self.async_add_entities(new_entities, update_before_add=update_before_add),
         )
 
-    def add_entities(self, new_entities, update_before_add=False):
+        if not self._setup_complete:
+            self._tasks.append(task)
+
+    def add_entities(
+        self, new_entities: Iterable[Entity], update_before_add: bool = False
+    ) -> None:
         """Add entities for a single platform."""
         # That avoid deadlocks
         if update_before_add:
@@ -258,7 +285,9 @@ class EntityPlatform:
             self.opp.loop,
         ).result()
 
-    async def async_add_entities(self, new_entities, update_before_add=False):
+    async def async_add_entities(
+        self, new_entities: Iterable[Entity], update_before_add: bool = False
+    ) -> None:
         """Add entities for a single platform async.
 
         This method must be run in the event loop.
@@ -272,7 +301,7 @@ class EntityPlatform:
         device_registry = await opp.helpers.device_registry.async_get_registry()
         entity_registry = await opp.helpers.entity_registry.async_get_registry()
         tasks = [
-            self._async_add_entity(
+            self._async_add_entity(  # type: ignore
                 entity, update_before_add, entity_registry, device_registry
             )
             for entity in new_entities
@@ -282,7 +311,24 @@ class EntityPlatform:
         if not tasks:
             return
 
-        await asyncio.wait(tasks)
+        timeout = max(SLOW_ADD_ENTITY_MAX_WAIT * len(tasks), SLOW_ADD_MIN_TIMEOUT)
+        try:
+            async with self.opp.timeout.async_timeout(timeout, self.domain):
+                await asyncio.gather(*tasks)
+        except asyncio.TimeoutError:
+            self.logger.warning(
+                "Timed out adding entities for domain %s with platform %s after %ds",
+                self.domain,
+                self.platform_name,
+                timeout,
+            )
+        except Exception:
+            self.logger.exception(
+                "Error adding entities for domain %s with platform %s",
+                self.domain,
+                self.platform_name,
+            )
+            raise
 
         if self._async_unsub_polling is not None or not any(
             entity.should_poll for entity in self.entities.values()
@@ -290,20 +336,22 @@ class EntityPlatform:
             return
 
         self._async_unsub_polling = async_track_time_interval(
-            self.opp, self._update_entity_states, self.scan_interval
+            self.opp,
+            self._update_entity_states,
+            self.scan_interval,
         )
 
-    async def _async_add_entity(
+    async def _async_add_entity(  # type: ignore[no-untyped-def]
         self, entity, update_before_add, entity_registry, device_registry
     ):
         """Add an entity to the platform."""
         if entity is None:
             raise ValueError("Entity cannot be None")
 
-        entity.opp = self.opp
-        entity.platform = self
-        entity.parallel_updates = self._get_parallel_updates_semaphore(
-            hasattr(entity, "async_update")
+        entity.add_to_platform_start(
+            self.opp,
+            self,
+            self._get_parallel_updates_semaphore(hasattr(entity, "async_update")),
         )
 
         # Update properties before we generate the entity_id
@@ -312,13 +360,16 @@ class EntityPlatform:
                 await entity.async_device_update(warning=False)
             except Exception:  # pylint: disable=broad-except
                 self.logger.exception("%s: Error on device update!", self.platform_name)
+                entity.add_to_platform_abort()
                 return
 
-        suggested_object_id = None
+        requested_entity_id = None
+        suggested_object_id: Optional[str] = None
 
         # Get entity_id from unique ID registration
         if entity.unique_id is not None:
             if entity.entity_id is not None:
+                requested_entity_id = entity.entity_id
                 suggested_object_id = split_entity_id(entity.entity_id)[1]
             else:
                 suggested_object_id = entity.name
@@ -327,7 +378,7 @@ class EntityPlatform:
                 suggested_object_id = f"{self.entity_namespace} {suggested_object_id}"
 
             if self.config_entry is not None:
-                config_entry_id = self.config_entry.entry_id
+                config_entry_id: Optional[str] = self.config_entry.entry_id
             else:
                 config_entry_id = None
 
@@ -342,8 +393,13 @@ class EntityPlatform:
                     "manufacturer",
                     "model",
                     "name",
+                    "default_manufacturer",
+                    "default_model",
+                    "default_name",
                     "sw_version",
+                    "entry_type",
                     "via_device",
+                    "suggested_area",
                 ):
                     if key in device_info:
                         processed_dev_info[key] = device_info[key]
@@ -383,6 +439,7 @@ class EntityPlatform:
                     or entity.name
                     or f'"{self.platform_name} {entity.unique_id}"',
                 )
+                entity.add_to_platform_abort()
                 return
 
         # We won't generate an entity ID if the platform has already set one
@@ -408,30 +465,46 @@ class EntityPlatform:
 
         # Make sure it is valid in case an entity set the value themselves
         if not valid_entity_id(entity.entity_id):
-            raise OpenPeerPowerError(f"Invalid entity id: {entity.entity_id}")
+            entity.add_to_platform_abort()
+            raise OpenPeerPowerError(f"Invalid entity ID: {entity.entity_id}")
 
         already_exists = entity.entity_id in self.entities
+        restored = False
 
-        if not already_exists:
+        if not already_exists and not self.opp.states.async_available(entity.entity_id):
             existing = self.opp.states.get(entity.entity_id)
-
-            if existing and not existing.attributes.get("restored"):
+            if existing is not None and ATTR_RESTORED in existing.attributes:
+                restored = True
+            else:
                 already_exists = True
 
         if already_exists:
-            msg = f"Entity id already exists: {entity.entity_id}"
             if entity.unique_id is not None:
-                msg += f". Platform {self.platform_name} does not generate unique IDs"
-            raise OpenPeerPowerError(msg)
+                msg = f"Platform {self.platform_name} does not generate unique IDs. "
+                if requested_entity_id:
+                    msg += f"ID {entity.unique_id} is already used by {entity.entity_id} - ignoring {requested_entity_id}"
+                else:
+                    msg += f"ID {entity.unique_id} already exists - ignoring {entity.entity_id}"
+            else:
+                msg = f"Entity id already exists - ignoring: {entity.entity_id}"
+            self.logger.error(msg)
+            entity.add_to_platform_abort()
+            return
 
         entity_id = entity.entity_id
         self.entities[entity_id] = entity
+
+        if not restored:
+            # Reserve the state in the state machine
+            # because as soon as we return control to the event
+            # loop below, another entity could be added
+            # with the same id before `entity.add_to_platform_finish()`
+            # has a chance to finish.
+            self.opp.states.async_reserve(entity.entity_id)
+
         entity.async_on_remove(lambda: self.entities.pop(entity_id))
 
-        await entity.async_internal_added_to_opp()
-        await entity.async_added_to_opp()
-
-        await entity.async_update_op_state()
+        await entity.add_to_platform_finish()
 
     async def async_reset(self) -> None:
         """Remove all entities and reset data.
@@ -445,13 +518,22 @@ class EntityPlatform:
         if not self.entities:
             return
 
-        tasks = [self.async_remove_entity(entity_id) for entity_id in self.entities]
+        tasks = [entity.async_remove() for entity in self.entities.values()]
 
-        await asyncio.wait(tasks)
+        await asyncio.gather(*tasks)
 
         if self._async_unsub_polling is not None:
             self._async_unsub_polling()
             self._async_unsub_polling = None
+        self._setup_complete = False
+
+    async def async_destroy(self) -> None:
+        """Destroy an entity platform.
+
+        Call before discarding the object.
+        """
+        await self.async_reset()
+        self.opp.data[DATA_ENTITY_PLATFORM][self.platform_name].remove(self)
 
     async def async_remove_entity(self, entity_id: str) -> None:
         """Remove entity id from platform."""
@@ -464,7 +546,9 @@ class EntityPlatform:
             self._async_unsub_polling()
             self._async_unsub_polling = None
 
-    async def async_extract_from_service(self, service_call, expand_group=True):
+    async def async_extract_from_service(
+        self, service_call: ServiceCall, expand_group: bool = True
+    ) -> List[Entity]:
         """Extract all known and available entities from a service call.
 
         Will return an empty list if entities specified but unknown.
@@ -476,15 +560,29 @@ class EntityPlatform:
         )
 
     @callback
-    def async_register_entity_service(self, name, schema, func, required_features=None):
-        """Register an entity service."""
+    def async_register_entity_service(self, name, schema, func, required_features=None):  # type: ignore[no-untyped-def]
+        """Register an entity service.
+
+        Services will automatically be shared by all platforms of the same domain.
+        """
+        if self.opp.services.has_service(self.platform_name, name):
+            return
+
         if isinstance(schema, dict):
             schema = cv.make_entity_service_schema(schema)
 
-        async def handle_service(call):
+        async def handle_service(call: ServiceCall) -> None:
             """Handle the service."""
             await service.entity_service_call(
-                self.opp, [self], func, call, required_features
+                self.opp,
+                [
+                    plf
+                    for plf in self.opp.data[DATA_ENTITY_PLATFORM][self.platform_name]
+                    if plf.domain == self.domain
+                ],
+                func,
+                call,
+                required_features,
             )
 
         self.opp.services.async_register(
@@ -518,9 +616,25 @@ class EntityPlatform:
                 tasks.append(entity.async_update_op_state(True))
 
             if tasks:
-                await asyncio.wait(tasks)
+                await asyncio.gather(*tasks)
 
 
 current_platform: ContextVar[Optional[EntityPlatform]] = ContextVar(
     "current_platform", default=None
 )
+
+
+@callback
+def async_get_platforms(
+    opp: OpenPeerPowerType, integration_name: str
+) -> List[EntityPlatform]:
+    """Find existing platforms."""
+    if (
+        DATA_ENTITY_PLATFORM not in opp.data
+        or integration_name not in opp.data[DATA_ENTITY_PLATFORM]
+    ):
+        return []
+
+    platforms: List[EntityPlatform] = opp.data[DATA_ENTITY_PLATFORM][integration_name]
+
+    return platforms
